# -*-  coding: utf-8 -*-
# Copyright (C) 2015 ZetaOps Inc.
#
# This file is licensed under the GNU General Public License v3
# (GPLv3).  See LICENSE.txt for details.

from datetime import time
from pyoko import Model, field, ListNode
from ulakbus.lib.date_time_helper import gun_dilimi_listele, HAFTA, gun_listele
from ulakbus.models import RoomType, Sube, Donem, Ders, Ogrenci
from zengine.forms import fields
from zengine.lib.translation import gettext_lazy as _, gettext, format_time, format_datetime
from .buildings_rooms import Room
from .auth import Unit
from .ogrenci import Okutman
import random


def uygunluk_durumu_listele():
    return [
        (1, gettext(u"Uygun")),
        (2, gettext(u"Mümkünse Uygun Değil")),
        (3, gettext(u"Kesinlikle Uygun Değil"))
    ]


def derslik_durumu_listele():
    return [
        (1, gettext(u'Herkese Açık')),
        (2, gettext(u'Bölüme Ait')),
        (3, gettext(u'Herkese Kapalı')),
    ]


class ZamanDilimleri(Model):
    class Meta:
        verbose_name = _(u"Zaman Dilimi")
        verbose_name_plural = _(u"Zaman Dilimleri")
        unique_together = [('birim', 'gun_dilimi')]
        search_fields = ['birim', 'gun_dilimi']

    birim = Unit(_(u'Bölüm'))
    gun_dilimi = field.Integer(_(u'Gün Dilimi'), choices=gun_dilimi_listele, index=True)

    baslama_saat = field.String(_(u"Başlama Saati"), index=True)
    baslama_dakika = field.String(_(u"Başlama Dakikası"), index=True)

    bitis_saat = field.String(_(u"Bitiş Saati"), index=True)
    bitis_dakika = field.String(_(u"Bitiş Dakikası"), index=True)

    # Ara suresi de dahil. Ornek olarak 30 girildiyse ders 9, 9.30, 10 gibi surelerde baslayabilir.
    ders_araligi = field.Integer(_(u'Ders Süresi'), default=60, index=True)
    ara_suresi = field.Integer(_(u'Tenefüs Süresi'), default=10, index=True)

    zaman_dilimi_suresi = field.Integer(_(u"Zaman Dilimi Süresi"), index=True)

    def pre_save(self):
        self.zaman_dilimi_suresi = int(self.bitis_saat) - int(self.baslama_saat)

    def __unicode__(self):
        return '%s - %s|%s' % (
            dict(gun_dilimi_listele())[int(self.gun_dilimi)],
            format_time(time(int(self.baslama_saat), int(self.baslama_dakika))),
            format_time(time(int(self.baslama_dakika), int(self.bitis_dakika))),
        )


class OgElemaniZamanPlani(Model):
    """
        Okutman, birim ve okutmanin ilgili birimde verecegi  haftalik ders saati bilgisi tutulur.
    """

    class Meta:
        verbose_name = _(u'Öğretim Elemanı Zaman Kaydı')
        verbose_name_plural = _(u'Öğretim Elemanı Zaman Kayıtları')
        unique_together = [('okutman', 'birim')]
        search_fields = ['okutman', 'birim']

    okutman = Okutman(_(u"Öğretim Elemanı"))
    birim = Unit(_(u"Birim"))
    toplam_ders_saati = field.Integer(_(u"Öğretim Elemanı Toplam Ders Saati"), index=True)

    def __unicode__(self):
        return '%s - %s' % (self.birim, self.okutman)


class ZamanCetveli(Model):
    """
        Ilgili birime ait belirlenen zaman dilimleri ders program koordinatoru tarafindan
        ogretim elemanlarin saat araliklarina gore durumlarini belirleyecegi model
    """

    class Meta:
        verbose_name = _(u'Zaman Cetveli')
        verbose_name_plural = _(u'Zaman Cetvelleri')

        unique_together = [('zaman_dilimi', 'ogretim_elemani_zaman_plani', 'gun')]
        search_fields = ['zaman_dilimi', 'ogretim_elemani_zaman_plani', 'birim', 'gun', 'durum']

    birim = Unit(_(u"Birim"))
    gun = field.Integer(_(u"Gün"), choices=gun_listele, index=True)
    zaman_dilimi = ZamanDilimleri(_(u"Zaman Dilimi"))
    durum = field.Integer(
        _(u"Uygunluk Durumu"),
        choices=uygunluk_durumu_listele,
        default=1, index=True
    )
    ogretim_elemani_zaman_plani = OgElemaniZamanPlani(_(u"Öğretim Elemanı"))

    def __unicode__(self):
        return '%s - %s - %s' % (self.ogretim_elemani_zaman_plani, self.zaman_dilimi,
                                 dict(uygunluk_durumu_listele())[int(self.durum)])


class DerslikZamanPlani(Model):
    class Meta:
        verbose_name = _(u'Derslik Zaman Planı')
        verbose_name_plural = _(u'Derslik Zaman Planları')

        unique_together = [
            ('derslik', 'gun', 'baslangic_saat', 'baslangic_dakika', 'bitis_saat', 'bitis_dakika')]
        search_fields = ['unit', 'derslik', 'gun', 'derslik_durum']

    birim = Unit()
    derslik = Room()
    gun = field.Integer(_(u"Gün"), choices=gun_listele, index=True)
    baslangic_saat = field.String(_(u'Başlangıç Saati'), default='08', index=True)
    baslangic_dakika = field.String(_(u'Başlangıç Dakikası'), default='30', index=True)
    bitis_saat = field.String(_(u"Bitiş Saati"), default='12', index=True)
    bitis_dakika = field.String(_(u"Bitiş Dakikası"), default='00', index=True)
    durum = field.Integer(_(u"Durum"), choices=derslik_durumu_listele, index=True)

    def __unicode__(self):
        return '%s %s %s|%s %s' % (
            self.derslik,
            dict(HAFTA)[self.gun],
            format_time(time(int(self.baslangic_saat), int(self.baslangic_dakika))),
            format_time(time(int(self.bitis_saat), int(self.bitis_dakika))),
            dict(derslik_durumu_listele())[int(self.derslik_durum)]
        )

    @staticmethod
    def kullanabilen_bolumler(derslik, **kwargs):
        units = set(DerslikZamanPlani.objects.filter(derslik=derslik, **kwargs).values_list('unit_id'))
        return units

    @staticmethod
    def kullanabildigi_derslikler(unit, make_object=True, **kwargs):
        rooms = set(DerslikZamanPlani.objects.filter(unit=unit, **kwargs).values_list('derslik_id'))
        if make_object:
            return map(Room.objects.get, rooms)
        else:
            return rooms


class DersEtkinligi(Model):
    class Meta:
        verbose_name = _(u"Ders Etkinliği")
        verbose_name_plural = _(u"Ders Etkinlikleri")
        search_fields = ['unit_yoksis_no', 'room', 'okutman']

    solved = fields.Boolean(_(u'Ders Planı Çözüm Durumu'), index=True)
    unitime_key = fields.String(index=True)  # class id
    unit_yoksis_no = fields.Integer(_(u'Bölüm Yöksis Numarası'), index=True)
    room_type = RoomType(_(u'İşleneceği Oda Türü'), index=True)
    okutman = Okutman(_(u"Öğretim Elemanı"), index=True)
    sube = Sube(_(u'Şube'), index=True)
    donem = Donem(_(u'Dönem'), index=True)
    bolum = Unit(_(u'Bölüm'), index=True)
    published = fields.Boolean(_(u'Ders Planı Yayınlanma Durumu'), index=True)
    # Arama amaçlı
    ders = Ders(_(u'Ders'), index=True)
    ek_ders = fields.Boolean(index=True)
    sure = fields.Integer("Ders Etkinliği Süresi", index=True)

    # teori = field.Integer("Ders Teori Saati", index=True)
    # uygulama = field.Integer("Ders Uygulama Saati", index=True)
    # dersin süresinin ne kadarı teori ne kadarı uygulama gibi 2+2, 4+0 gibi

    # to be calculated
    room = Room(_(u'Derslik'))
    gun = fields.String(_(u"Gün"), choices=gun_listele)
    baslangic_saat = fields.String(_(u"Başlangıç Saati"))
    baslangic_dakika = fields.String(_(u"Başlangıç Dakikası"))
    bitis_saat = fields.String(_(u"Bitiş Saati"))
    bitis_dakika = fields.String(_(u"Bitiş Dakikası"))

    def post_creation(self):
        """Yeni bir DersEtkinligi oluşturulduğunda arama amaçlı olan
        alanları otomatik olarak doldurur."""
        self.ders = self.sube.ders
        self.save()

    def __unicode__(self):
        return '%s - %s - %s:%s|%s:%s - %s' % (
            self.room.name, self.gun, self.baslangic_saat, self.baslangic_dakika,
            self.bitis_saat, self.bitis_dakika, self.okutman)


class SinavEtkinligi(Model):
    class Meta:
        verbose_name = _(u'Sınav Etkinliği')
        verbose_name_plural = _(u'Sınav Etkinlikleri')
        search_field = ['bolum', 'ders', 'sube', 'donem']

    sube = Sube(_(u'Şube'), index=True)
    ders = Ders(_(u'Ders'), index=True)
    donem = Donem(_(u'Dönem'), index=True)
    bolum = Unit(_(u'Bölüm'), index=True)
    unitime_key = fields.String(index=True)
    # default False, unitime solver tarafindan True yapilir.
    solved = fields.Boolean(_(u'Sınav Planı Çözüm Durumu'), index=True, default=False)

    # unitime cozumunun ardindan, is akisiyla sinav takvimi yayinlanip True yapilir.
    published = fields.Boolean(_(u'Sınav Planı Yayınlanma Durumu'), index=True, default=False)

    # sistem servisiyle sinavlarin ardindan True yapilir.
    archived = fields.Boolean(_(u'Arşivlenmiş'), default=False, index=True)

    tarih = fields.DateTime(_(u'Sınav Tarihi'), index=True)

    class SinavYerleri(ListNode):
        room = Room(_(u'Sınav Yeri'), index=True)

    class Ogrenciler(ListNode):
        ogrenci = Ogrenci(_(u'Öğrenci'))
        room = Room(_(u'Sınav Yeri'), index=True)

    @classmethod
    def aktif_bolum_sinav_etkinlik_listesi(cls, donem, bolum):
        """
        Verilen bölümün aktif yayınlanmış sınav etkinliklerinin
        listesini döndürür.

        """
        return [e for e in
                cls.objects.filter(published=True, archived=False, donem=donem, bolum=bolum)]

    def sinav_ogrenci_listesi(self):
        """
        Verilen sınav etkinliğine katılacak olan öğrencilerin
        listesini döndürür.

        """
        return [e.ogrenci for e in self.Ogrenciler]

    def doluluk_orani_hesapla(self):
        """
        Bir sınav etkinliğine kayıtlı olan öğrencilerin sayısı ile
        etkinliğin yapılacak sınav yerlerinin toplam kontenjan sayısının
        birbirine bölünmesi ile elde edilen oranı döndürür. Bu oran öğrencileri
        odalara dengeli şekilde dağıtmak için kullanılacaktır.

        """
        toplam_kontenjan = sum([e.room.capacity for e in self.SinavYerleri])
        doluluk_orani = len(self.Ogrenciler) / float(toplam_kontenjan)

        return doluluk_orani

    def ogrencileri_odalara_dagit(self, doluluk_orani):
        """
        Öğrencileri sınavın yapılacağı odalara doluluk oranını
        göz önüne alarak dengeli bir şekilde dağıtır.

        """
        from math import ceil
        random.shuffle(self.Ogrenciler)
        j = 0
        for sinav_yeri in self.SinavYerleri:
            temp = j + int(ceil(sinav_yeri.room.capacity * doluluk_orani))
            i = j
            j = temp
            for ogrenci in self.Ogrenciler[i:j]:
                ogrenci.room = sinav_yeri.room

        self.save()

    @classmethod
    def ogrencileri_odalara_rastgele_ata(cls, bolum):
        """
        Bir bölümün yayınlanmış sınav programındaki her bir sınav etkinliğine
        katılacak olan öğrencileri, sınavın yapılabilineceği odalara rastgele
        atar, bu atamayı yaparken kontenjan sınırını aşmamasına dikkat edilir.

        """
        donem = Donem.guncel_donem()
        aktif_sinav_etkinlikleri = cls.aktif_bolum_sinav_etkinlik_listesi(donem, bolum)
        for etkinlik in aktif_sinav_etkinlikleri:
            doluluk_orani = etkinlik.doluluk_orani_hesapla()
            etkinlik.ogrencileri_odalara_dagit(doluluk_orani)

    def ogrenci_sinav_oda_getir(self, ogrenci):
        """
        Verilen öğrencinin sınava gireceği oda bilgisini döndürür.

        """
        for ogrenci_oda in self.Ogrenciler:
            if ogrenci_oda.ogrenci == ogrenci:
                break

        return ogrenci_oda.room

    @classmethod
    def sube_sinav_listesi(cls, sube, archived=False, donem=None):
        """
        Şubeye, döneme ve arşive göre sınav etkinliği listesi döndürür.

        """
        donem = donem or Donem.guncel_donem()
        return [e for e in cls.objects.filter(
            published=True,
            sube=sube,
            archived=archived,
            donem=donem
        ).order_by('-tarih')]

    def __unicode__(self):
        return '{} {} {}'.format(self.ders.ad, self.sube.ad, self.sinav_zamani())

    def sinav_zamani(self):
        return format_datetime(self.tarih) if self.tarih else _(u'Henüz zamanlanmadi')


class SinavZamanDilimi(Model):
    class Meta:
        verbose_name = _(u'Sınav Zaman Dilimi')
        verbose_name_plural = _(u'Sınav Zaman Dilimleri')
        unique_together = [('birim', 'gun_dilimi')]
        search_fields = ['birim', 'gun_dilimi']

    birim = Unit(_(u'Bölüm'))
    gun_dilimi = field.Integer(_(u'Gün Dilimi'), choices=gun_dilimi_listele)
    baslama_saat = field.String(_(u"Başlama Saati"))
    baslama_dakika = field.String(_(u"Başlama Dakikası"))

    bitis_saat = field.String(_(u"Bitiş Saati"))
    bitis_dakika = field.String(_(u"Bitiş Dakikası"))

    def __unicode__(self):
        return '{}/{}:{}-{}:{}'.format(dict(gun_dilimi_listele())[self.gun_dilimi], self.baslama_saat, self.baslama_dakika,
                                       self.bitis_saat, self.bitis_dakika)


class DerslikSinavPlani(Model):
    class META:
        verbose_name = _(u'Derslik Sınav Planı')
        verbose_name_plural = _(u'Derslik Sınav Planları')
        unique_together = [('derslik', 'zaman_dilimi', 'birim', 'gun')]
        search_fields = ['derslik', 'birim', 'zaman_dilimi', 'gun', 'durum']

    derslik = Room(_(u'Derslik'))
    birim = Unit(_(u'Bölüm'))
    donem = Donem(_(u'Güncel Dönem'))
    zaman_dilimi = SinavZamanDilimi(_(u'Sınav Zaman Dilimi'))
    durum = field.Integer(_(u"Uygunluk Durumu"), choices=uygunluk_durumu_listele(), default=1, index=True)
    gun = field.Integer(_(u"Gün"), choices=HAFTA)

    def __unicode__(self):
        return '{}/{}/{}'.format(dict(HAFTA)[self.gun], self.zaman_dilimi,
                                 dict(uygunluk_durumu_listele())[self.durum])



# -*-  coding: utf-8 -*-
"""
"""

# Copyright (C) 2015 ZetaOps Inc.
#
# This file is licensed under the GNU General Public License v3
# (GPLv3).  See LICENSE.txt for details.
from zengine.lib.cache import Cache


class RolePermissionCache(Cache):
    PREFIX = 'PERM'

    def __init__(self, role_id):
        super(RolePermissionCache, self).__init__(role_id)


class GuncelDonem(Cache):
    """
    Güncel dönemin tutulduğu cache nesnesi.
    """

    PREFIX = 'GUNDON'

    def __init__(self):
        super(GuncelDonem, self).__init__('guncel_donem')

    def get_or_set(self):
        """
        Cache'de güncel dönem datası varsa döndürür.
        Yoksa veritabanından güncel dönemi alıp cache'e set eder.
        Güncel dönem objesi dönemlik değişeceği için, set edilirken
        expire olma süresi 360000 yani 100 saat olarak belirlenmiştir.

        Returns: cache_data(dict): Güncel dönem nesnesinin fieldlarının dict hali.

        """

        from ulakbus.models.ogrenci import Donem

        cache_data = self.get()

        if not cache_data:
            guncel_donem = Donem.objects.get(guncel=True)
            cache_data = guncel_donem.donem_fields_to_dict()
            self.set(cache_data, 360000)

        return cache_data



# -*-  coding: utf-8 -*-
"""
"""

# Copyright (C) 2015 ZetaOps Inc.
#
# This file is licensed under the GNU General Public License v3
# (GPLv3).  See LICENSE.txt for details.
#
from collections import OrderedDict

from ulakbus.models import DersEtkinligi, Room, Donem
from zengine.forms import JsonForm, fields
from zengine.views.crud import CrudView
from ulakbus.lib.date_time_helper import map_etkinlik_hafta_gunleri, HAFTA
from zengine.lib.translation import gettext as _, gettext_lazy


class DerslikSecimFormu(JsonForm):
    """
    Derslik Seç adımında kullanılan formdur.

    """
    ileri = fields.Button(gettext_lazy(u"İleri"))


class DerslikDersProgrami(CrudView):
    """ Derslik Ders Programı İş Akışı
    Derslik Ders Programi,bir bölümün sahip olduğu dersliklere ait ders programlarını gösterir ve
    kullanıcının ders programlarını yazdırabilmesine olanak sağlar.

    Bu iş akışı 4 adımdan oluşur.

    Ders Etkinliklerini Kontrol Et:
    Yayınlanmış ders etkinlikleri var mı yok mu diye kontrol eder.

    Bilgi Ver:
    Yayınlanmış ders etkinlikleri yok ise ekrana bilgi mesajı basılır.

    Derslik Seç:
    Derslikler listelenir.

    Derslik Ders Programını Göster:
    Seçilen dersliğe ait ders programlarını ekrana basar.

    """
    class Meta:
        model = 'DersEtkinligi'

    def ders_etkinliklerini_kontrol_et(self):
        """
        Yayınlanmış ders etkinlikleri var mı yok mu diye kontrol eder.

        """
        length = len(DersEtkinligi.objects.filter(published=True,
                                                  donem=Donem.guncel_donem(self.current),
                                                  bolum=self.current.role.unit))
        self.current.task_data['yayinlanmamis_ders_sayisi'] = length

    def bilgi_ver(self):
        """
        Yayınlanmış ders etkinlikleri yok ise  ekrana bilgi mesajı basılır.

        """
        self.current.output['msgbox'] = {
            'type': 'info', "title": _(u'Yayınlanmamış Dersler'),
            'msg': _(u"Yayınlanmış dersler bulunmamaktadır.")
        }

    def derslik_sec(self):
        """
        Derslikler listelenir.

        """
        _form = DerslikSecimFormu(title=_(u'Derslik Seçiniz'), current=self.current)
        ders_etkinlikleri = DersEtkinligi.objects.filter(published=True,
                                                         donem=Donem.guncel_donem(self.current),
                                                         bolum=self.current.role.unit)
        _choices = [(_etkinlik.room.key, _etkinlik.room.__unicode__())
                    for _etkinlik in ders_etkinlikleri]
        _form.derslik = fields.Integer(choices=_choices)
        self.form_out(_form)

    def derslik_ders_programini_goster(self):
        """
        Seçilen dersliğe ait ders programlarını ekrana basar.

        """
        room = Room.objects.get(self.current.input['form']['derslik'])
        hafta = dict(HAFTA)
        self.output['objects'] = [hafta]
        d_etkinlikleri = DersEtkinligi.objects.filter(room=room,
                                                      published=True,
                                                      donem=Donem.guncel_donem(self.current))
        ders_etkinlikleri = map_etkinlik_hafta_gunleri(
            d_etkinlikleri.order_by(
                'gun', 'baslangic_saat',
                'bitis_saat','baslangic_dakika', 'bitis_dakika'))
        # Bir güne ait maximum etkinlik sayısı.
        max_etkinlik = max(map(len, ders_etkinlikleri.values()))
        for i in range(max_etkinlik):
            ders_etkinlikleri_dict = OrderedDict({})
            for hafta_gun in hafta.keys():
                if hafta_gun in ders_etkinlikleri:
                    try:
                        etkinlik = ders_etkinlikleri[hafta_gun][i]
                        ders_etkinlikleri_dict[hafta[hafta_gun]] = etkinlik
                    except IndexError:
                        ders_etkinlikleri_dict[hafta[hafta_gun]] = ""
                else:
                    ders_etkinlikleri_dict[hafta[hafta_gun]] = ""
            item = {
                "type": "table-multiRow",
                "fields": ders_etkinlikleri_dict,
                "actions": False,
                'key': ''
            }
            self.output['objects'].append(item)

        _form = JsonForm(title=_(u"%s Dersliğine Ait Ders Programları") % room.__unicode__())
        _form.yazdir = fields.Button(_(u'Pdf yazdır'))
        self.form_out(_form)
        self.current.output["meta"]["allow_actions"] = False



# -*-  coding: utf-8 -*-

# Copyright (C) 2015 ZetaOps Inc.
#
# This file is licensed under the GNU General Public License v3
# (GPLv3).  See LICENSE.txt for details.

"""HITAP Açık Süre Sorgula

Hitap üzerinden personelin
açık süre hizmet bilgilerinin sorgulamasını yapar.

"""

from ulakbus.services.personel.hitap.hitap_sorgula import HITAPSorgula


class HizmetAcikSureGetir(HITAPSorgula):
    """
    HITAP Sorgulama servisinden kalıtılmış
    Açık Süre Hizmet Bilgisi Sorgulama servisi

    """
    HAS_CHANNEL = True

    def handle(self):
        """
        Servis çağrıldığında tetiklenen metod.

        Attributes:
            service_name (str): İlgili Hitap sorgu servisinin adı
            bean_name (str): Hitap'tan gelen bean nesnesinin adı
            service_dict (dict): Hitap servisinden gelen kayıtların alanları,
                    ``HizmetAcikSure`` modelinin alanlarıyla eşlenmektedir.
                    Filtreden geçecek tarih alanları listede tutulmaktadır.

        """

        self.service_name = 'HizmetAcikSureSorgula'
        self.bean_name = 'HizmetAcikSureServisBean'
        self.service_dict = {
            'fields': {
                'tckn': 'tckn',
                'kayit_no': 'kayitNo',
                'acik_sekil': 'acikSekil',
                'iade_sekil': 'iadeSekil',
                'hizmet_durum': 'hizmetDurum',
                'husus': 'husus',
                'aciga_alinma_tarih': 'acigaAlinmaTarih',
                'goreve_son_tarih': 'goreveSonTarih',
                'goreve_iade_istem_tarih': 'goreveIadeIstemTarih',
                'goreve_iade_tarih': 'goreveIadeTarih',
                'acik_aylik_bas_tarih': 'acikAylikBasTarih',
                'acik_aylik_bit_tarih': 'acikAylikBitTarih',
                'goreve_son_aylik_bas_tarih': 'goreveSonAylikBasTarih',
                'goreve_son_aylik_bit_tarih': 'goreveSonAylikBitTarih',
                's_yonetim_kald_tarih': 'SYonetimKaldTarih',
                'aciktan_atanma_tarih': 'aciktanAtanmaTarih',
                'kurum_onay_tarihi': 'kurumOnayTarihi'
            },
            'date_filter': ['aciga_alinma_tarih', 'goreve_son_tarih', 'goreve_iade_istem_tarih',
                            'goreve_iade_tarih', 'acik_aylik_bas_tarih', 'acik_aylik_bit_tarih',
                            'goreve_son_aylik_bas_tarih', 'goreve_son_aylik_bit_tarih',
                            's_yonetim_kald_tarih', 'aciktan_atanma_tarih', 'kurum_onay_tarihi'],
            'required_fields': ['tckn']
        }
        super(HizmetAcikSureGetir, self).handle()

    def custom_filter(self, hitap_dict):
        """
        Hitap sözlüğüne uygulanacak ek filtreleri gerçekleştirir.

        Args:
            hitap_dict (List[dict]): Hitap verisini yerele uygun biçimde tutan sözlük listesi

        """

        for record in hitap_dict:
            husus = record['husus']
            record['husus'], record['husus_aciklama'] = self.husus_aciklama_kontrol(husus)

    def husus_aciklama_kontrol(self, husus):
        """
        Hitap Açık Süre servisinin,
        husus bilgisi tek başına husus kodunu veya
        husus kodu ve açıklamasını veya
        husus kodu ve birkaç tarih bilgisini birlikte içerebilmektedir.

        Bu amaçla husus kodu ve açıklama (mahkeme detayı, tarih bilgileri vs.)
        kısımları ayrı ayrı elde edilmektedir.

        Args:
            husus (str): Açık Süre husus bilgisi.
                Gelen veri 1, 2, 3, 4, 5, 6, 7, 10, 11, 14 değerlerinden biriyse,
                sadece husus kodunu içermektedir. Doğrudan husus kodu olarak elde edilir.
                Gelen veri 8, 9, 12, 13, 15, 16, 17, 18 değerleriyle birlikte
                açıklama (mahkeme detayı, tarih bilgileri vs.) kısmını da içeriyorsa,
                husus kodu ve açıklama olacak şekilde iki parçalı olarak elde edilir.

        Returns:
            (int, str): Husus kodu ve açıklaması.

        Raises:
            IndexError: Husus açıklaması eksik.

        """

        husus = husus.split(' ', 1)
        try:
            husus_kodu = int(husus[0])
        except ValueError:
            self.logger.exception("Husus Kodu tam sayi olmali.")
            return 0, ""

        # only code
        if husus_kodu in [1, 2, 3, 4, 5, 6, 7, 10, 11, 14]:
            return husus_kodu, ""
        # code and explanation
        elif husus_kodu in [8, 9, 12, 13, 15, 16, 17, 18]:
            try:
                aciklama = husus[1]
                return husus_kodu, aciklama
            except IndexError:
                self.logger.exception("Husus aciklamasi yok.")
                return husus_kodu, ""
        else:
            self.logger.info('Husus Kodu gecersiz.')
            return 0, ""



# -*-  coding: utf-8 -*-

# Copyright (C) 2015 ZetaOps Inc.
#
# This file is licensed under the GNU General Public License v3
# (GPLv3).  See LICENSE.txt for details.


from zengine.views.crud import CrudView
from zengine.forms import JsonForm, fields
from zengine.lib.translation import gettext as _, format_time
from ulakbus.models.ders_sinav_programi import ZamanDilimleri
from collections import OrderedDict
from datetime import time


class ZamanDilimiDuzenle(CrudView):

    def varsayilan_zaman_dilimleri(self):
        zaman_dilimleri = sorted(ZamanDilimleri.objects.filter(birim=self.current.role.unit),
                                 key=lambda zd: zd.gun_dilimi)

        self.output['objects'] = [[_(u'Gün Dilimi'), _(u'Zaman Aralığı')]]
        for data in zaman_dilimleri:
            data_list = OrderedDict({})
            data_list["Gün Dilimi"] = "%s" % (data.get_gun_dilimi_display())
            data_list["Zaman Aralığı"] = "%(baslama)s-%(bitis)s" % {
                'baslama': format_time(time(int(data.baslama_saat), int(data.baslama_dakika))),
                'bitis': format_time(time(int(data.bitis_saat), int(data.bitis_dakika))),
            }
            item = {
                "title": _(u"Varsayılan Zaman Dilimleri"),
                'type': "table-multiRow",
                'fields': data_list,
                'actions': [
                    {'name': _(u'Değiştir'), 'cmd': 'degistir', 'show_as': 'button', 'object_key':'zaman_dilimi'}
                ],
                'key': data.key}
            self.output['objects'].append(item)

        _json = JsonForm(title=_(u"Güncel Zaman Dilimleri"))
        _json.tamamla = fields.Button(_(u"Bitir"), cmd='tamamla')
        self.form_out(_json)

    def zaman_dilimlerini_belirle(self):
        self.current.task_data['zaman_dilimi'] = self.current.input['zaman_dilimi']
        zd = ZamanDilimleri.objects.get(self.current.input['zaman_dilimi'])
        _json = JsonForm(title=_(u"ZAMAN DİLİMİ"))
        _json.gun_dilimi = fields.String(title=_(u"GÜN DİLİMİ"), default=zd.get_gun_dilimi_display())
        _json.baslangic_saat = fields.String(_(u"Başlangıç Saati"), default=zd.baslama_saat, required=False)
        _json.baslangic_dakika = fields.String(_(u"Başlangıç Dakikası"), default=zd.baslama_dakika, required=False)
        _json.bitis_saat = fields.String(_(u"Bitiş Saati"), default=zd.bitis_saat, required=False)
        _json.bitis_dakika = fields.String(_(u"Bitiş Dakikası"), default=zd.bitis_dakika, required=False)
        _json.kaydet = fields.Button(_(u"Kaydet"), cmd='kayit')
        _json.tamamlandi = fields.Button(_(u'İptal'), cmd='vazgec')
        self.form_out(_json)

    def kaydet(self):
        zd = ZamanDilimleri.objects.get(self.current.task_data['zaman_dilimi'])
        try:
            zd.baslama_saat = self.input['form']['baslangic_saat']
            zd.baslama_dakika = self.input['form']['baslangic_dakika']
            zd.bitis_saat = self.input['form']['bitis_saat']
            zd.bitis_dakika = self.input['form']['bitis_dakika']
            zd.save()

            msg = {"type": "info",
                   "title": _(u'Kaydedildi!'),
                   "msg": _(u"Kaydınız başarıyla gerçekleşti")}

            self.current.output['msgbox'] = msg
        except:
            msg = {"type": "warning",
                   "title": _(u'Kayıt Başarısız Oldu!'),
                   "msg": _(u"Malesef kaydınız başarısız oldu")}

            self.current.output['msgbox'] = msg

    def kayit_islemi_tamamlandi(self):
        msg = {"type": "info",
               "title": _(u'Kayıt İşleminiz Tamamlanmıştır!'),
               "msg": _(u'Güncel Zaman Dilimleri Zaman Tablosuna eklenmiştir')}
        # workflowun bu kullanıcı için bitişinde verilen mesajı ekrana bastırır

        self.current.output['msgbox'] = msg



# -*-  coding: utf-8 -*-
"""Öğrenci ile ilgili yardımcı class ve fonksiyonların yer aldığı dosyadır.
"""

# Copyright (C) 2015 ZetaOps Inc.
#
# This file is licensed under the GNU General Public License v3
# (GPLv3).  See LICENSE.txt for details.

from enum import Enum
from operator import attrgetter
from ulakbus.lib.role import AbsRole
from ulakbus.models import AbstractRole


def diploma_no_uret(ogrenci_program):
    """
    Öğrenci programı için diploma no üretir.

    Args:
        ogrenci_program (): öğrenci program nesnesi

    Returns:
        diploama no (string)

    """
    return "%s-%s-%s" % (ogrenci_program.giris_tarihi,
                         ogrenci_program.program.yoksis_no, ogrenci_program.ogrenci_no)


def aktif_sinav_listesi(obj):
    """
    obj (öğrenci veya okutman) için aktif sınavlarının listesini üretir.

    Args:
       obj (Ogrenci): öğrenci veya okutman nesnesi
    Returns:
        sinav listesi (list)
    """

    from ulakbus.models.ders_sinav_programi import SinavEtkinligi
    sinavlar = []
    for sube in obj.donem_subeleri():
        sinavlar.extend(SinavEtkinligi.sube_sinav_listesi(sube=sube))
    return sinavlar


def kaydi_silinmis_abs_role(role):
    """
    Kaydı silinecek kaydın abstract rolünü getirir.
    Args:
        role: role nesnesi

    Returns:
        Abstract Role nesnesi

    """
    func = attrgetter("unit.unit_type", "unit.learning_duration")
    unit_type, learning_duration = func(role)

    if unit_type == "Program" and learning_duration == 4:
        return AbstractRole.objects.get(AbsRole.LISANS_OGRENCISI_KAYIT_SILINMIS.name)
    elif unit_type == "Program" and learning_duration == 2:
        return AbstractRole.objects.get(AbsRole.ON_LISANS_OGRENCISI_KAYIT_SILINMIS.name)
    elif unit_type == "Yüksek Lisans Programı":
        return AbstractRole.objects.get(AbsRole.YUKSEK_LISANS_OGRENCISI_KAYIT_SILINMIS.name)
    elif unit_type == "Doktora Programı":
        AbstractRole.objects.get(AbsRole.DOKTORA_OGRENCISI_KAYIT_SILINMIS.name)
    else:
        # TODO: Boş bir abstract role nesnesi yaratılmalı
        return AbstractRole()


def kaydi_dondurulmus_abs_role(role):
    """
    Dondurulacak kaydın abstract rolünü getirir.

    Args:
        role: role nesnesi

    Returns:
        Abstract Role nesnesi

    """
    func = attrgetter("unit.unit_type", "unit.learning_duration")
    unit_type, learning_duration = func(role)
    if unit_type == "Program" and learning_duration == 4:
        return AbstractRole.objects.get(AbsRole.LISANS_OGRENCISI_KAYIT_DONDURMUS.name)
    elif unit_type == "Program" and learning_duration == 2:
        return AbstractRole.objects.get(AbsRole.ON_LISANS_OGRENCISI_KAYIT_DONDURMUS.name)
    elif unit_type == "Yüksek Lisans Programı":
        return AbstractRole.objects.get(AbsRole.YUKSEK_LISANS_OGRENCISI_KAYIT_DONDURMUS.name)
    elif unit_type == "Doktora Programı":
        return AbstractRole.objects.get(AbsRole.DOKTORA_OGRENCISI_KAYIT_DONDURMUS.name)
    else:
        # TODO: Boş bir abstract role nesnesi yaratılmalı.
        return AbstractRole()


class HarfNotu(Enum):
    """
    HarfNotu.AA._4()

    """

    AA = {
        "name": "AA",
        "bas": 90,
        "bit": 100,
        "dort": 4.00
    }

    BA = {
        "name": "BA",
        "bas": 85,
        "bit": 89,
        "dort": 3.50
    }

    BB = {
        "name": "BB",
        "bas": 75,
        "bit": 84,
        "dort": 3.00
    }
    CB = {
        "name": "CB",
        "bas": 70,
        "bit": 74,
        "dort": 2.50
    }
    CC = {
        "name": "CC",
        "bas": 60,
        "bit": 69,
        "dort": 2.00
    }
    DC = {
        "name": "DC",
        "bas": 55,
        "bit": 59,
        "dort": 1.50
    }
    DD = {
        "name": "DD",
        "bas": 50,
        "bit": 54,
        "dort": 1.00
    }
    FD = {
        "name": "FD",
        "bas": 40,
        "bit": 49,
        "dort": 0.50
    }
    FF = {
        "name": "FF",
        "bas": 0,
        "bit": 39,
        "dort": 0.00
    }

    def get_4(self):
        return self.value.get('dort', None)

    def get_100(self):
        return self.value.get('bas', None), self.value.get('bit', None)

    @classmethod
    def puan_harf_notu(cls, puan):
        for name, obj in cls.__members__.items():
            bas, bit = obj.get_100()
            if puan in range(bas, bit + 1):
                return name

    @classmethod
    def generate_choices_for_4(cls):
        """
        4 luk notlar icin choices uretir

        Returns:
            list of tuples
        """
        return [(obj.get_4(), obj.get_4()) for name, obj in cls.__members__.items()]

    @classmethod
    def generate_choices(cls):
        """
        Harf notlari icin choices uretir

        Returns:
            list of tuples
        """
        return [(name, name) for name, obj in cls.__members__.items()]


class AkademikTakvimEtkinlikleri(Enum):
    YENI_OGRENCI_ON_KAYIT = "Yeni Öğrenci Ön Kayıt"
    GUZ_BASLANGICI = "Güz Dönem Başlangıcı"
    GUZ_DERSLERIN_ACILMASI = "Derslerin Acılması"
    GUZ_SUBELENDIRME_DERS_PROGRAMI_ILANI = "Subelendirme ve Ders Programının Ilan Edilmesi"
    GUZ_OGRENCI_HARC = "Öğrenci Harç"
    GUZ_OGRENCI_EK_HARC = "Oğrenci Ek Harç"
    GUZ_MAZERETLI_OGRENCI_HARC = "Mazeretli Öğrenci Harç"
    YENI_OGRENCI_DERS_KAYIT = "Yeni Öğrenci Ders Kayıt"
    YENI_OGRENCI_DANISMAN_ONAY = "Yeni Öğrenci Danışman Onay"
    GUZ_DERS_KAYIT = "Ders Kayıt"
    GUZ_DANISMAN_ONAY = "Danışman Onay"
    GUZ_MAZERETLI_DERS_KAYIT = "Mazeretli Ders Kayıt"
    GUZ_MAZERETLI_DANISMAN_ONAY = "Mazeretli Danışman Onay"
    GUZ_DERSLERIN_BASLANGICI = "Derslerin Başlangıcı"
    GUZ_DERS_EKLE_BIRAK = "Ders Ekle/Bırak"
    GUZ_DERS_EKLE_BIRAK_DANISMAN_ONAY = "Ders Ekle/Bırak Danışman Onay"
    GUZ_DANISMAN_DERSTEN_CEKILME_ISLEMLERI = "Danışman Dersten Çekilme İşlemleri"
    GUZ_ARA_SINAV = "Ara Sinav"
    GUZ_ARA_SINAV_NOT_GIRIS = "Ara Sınav Not Giriş"
    GUZ_ARA_SINAV_NOT_YAYINLAMA = "Ara Sınav Notlarının Öğrenciye Yayınlanması"
    GUZ_ARA_SINAV_MAZERETLI = "Ara Sinav Mazeretli"
    GUZ_ARA_SINAV_MAZERET_NOT_GIRIS = "Ara Sınav Mazeret Not Giriş"
    GUZ_ARA_SINAV_MAZERET_NOT_YAYINLAMA = "Ara Sınav Mazeret Notlarının Yayınlanması"
    GUZ_SINAV_MADDI_HATA_DUZELTME = "Sınav Maddi Hata Düzeltme"
    GUZ_DERSLERIN_BITISI = "Derslerin Bitişi"
    GUZ_YARIYIL_SINAV = "Yariyil Sinav"
    GUZ_YARIYIL_SINAVI_NOT_GIRIS = "Yarıyıl Sınavı Not Giriş"
    GUZ_YARIYIL_SINAVI_NO_YAYINLAMA = "Yarıyıl Sınavı Notlarının Öğrenciye Yayınlanmasi"
    GUZ_BUT_VE_YARIYIL_SONU_MAZERET_SINAVI = "Bütünleme ve Yarı Yıl Sonu Mazeret Sınavı"
    GUZ_BUT_VE_YARIYIL_SONU_MAZERET_SINAVI_NOT_GIRIS = "Bütünleme ve \
                                                        Yarı Yıl Sonu Mazeret Sınavı Not Giriş"
    GUZ_BUT_VE_YARIYIL_SONU_MAZERET_SINAVI_NOT_YAYINLAMA = "Bütünleme ve Yarı Yıl Sonu Mazeret \
                                                         Sınavı Notlarının Öğrenciye Yayınlanması"
    GUZ_HARF_NOT_YAYINLAMA = "Harf Notlarının Öğrenciye Yayınlanması"
    GUZ_BUT_HARF_NOT_YAYINLAMA = "Bütünleme Harf Notlarının Öğrenciye Yayınlanması"
    GUZ_OGRETIM_ELEMANI_YOKLAMA_GIRISI = "Öğretim Elemanı Yoklama Girişi"
    GUZ_DONEMI_BITIS = "Güz Dönemi Bitiş"
    BAHAR_BASLANGICI = "Bahar Dönemi Başlangıcı"
    BAHAR_DERSLERIN_ACILMASI = "Bahar Donemi Derslerin Acilmasi"
    BAHAR_SUBELENDIRME_DERS_PROGRAMI_ILANI = "Subelendirme ve Ders Programının Ilan Edilmesi"
    BAHAR_OGRENCI_HARC = "Öğrenci Harç"
    BAHAR_OGRENCI_EK_HARC = "Öğrenci Ek Harç"
    BAHAR_MAZERETLI_OGRENCI_HARC = "Mazeretli Öğrenci Harç"
    BAHAR_DERS_KAYIT = "Ders Kayıt"
    BAHAR_DANISMAN_ONAY = "Danışman Onay"
    BAHAR_MAZERETLI_DERS_KAYIT = "Mazeretli Ders Kayıt"
    BAHAR_MAZERETLI_DANISMAN_ONAY = "Mazeretli Danışman Onay"
    BAHAR_DERSLERIN_BASLANGICI = "Derslerin Başlangıcı"
    BAHAR_DERS_EKLE_BIRAK = "Ders Ekle / Bırak"
    BAHAR_DERS_EKLE_BIRAK_ONAY = "Ders Ekle / Bırak Onay"
    BAHAR_ARA_SINAV = "Ara Sinav"
    BAHAR_ARA_SINAV_NOT_GIRIS = "Ara Sınav Not Giriş"
    BAHAR_ARA_SINAV_NOTLARININ_OGRENCIYE_YAYINLANMASI = "Ara Sınav Notlarının Yayınlanması"
    BAHAR_ARA_SINAV_MAZERETLI = "Ara Sinav Mazeretli"
    BAHAR_ARA_SINAV_MAZERET_NOT_GIRIS = "Ara Sınav Mazeret Not Giriş"
    BAHAR_ARA_SINAV_MAZERET_NOT_YAYINLAMA = "Ara Sınav Mazeret Notlarının Öğrenciye Yayınlanması"
    BAHAR_SINAV_MADDI_HATA_DUZELTME = "Sınav Maddi Hata Düzeltme"
    BAHAR_DERSLERIN_BITISI = "Derslerin Bitişi"
    BAHAR_YARIYIL_SINAV_BASLANGIC = "Yariyil Sinav baslangic"
    BAHAR_YARIYIL_SINAVI_NOT_GIRIS = "Yarıyıl Sınavı Not Giriş"
    BAHAR_YARIYIL_SINAVI_NOT_YAYINLAMA = "Yarıyıl Sınavı Notlarının Öğrenciye Yayınlanmasi"
    BAHAR_OGRETIM_ELEMANI_YOKLAMA_GIRISI = "Öğretim Elemanı Yoklama Girişi"
    BAHAR_BITISI = "Bahar Dönem Bitişi"
    YAZ_DONEMI_BASLANGICI = "Yaz Dönemi Başlangıcı"
    YAZ_DONEMI_DERSLERIN_BITISI = "Yaz Dönemi Derslerin Bitişi"
    YAZ_DONEMI_SINAVLARIN_BASLANGICI = "Yaz Dönemi Sınavların Başlangıcı"
    YAZ_DONEMI_BITISI = "Yaz Dönemi Bitişi"
    GUZ_DONEMI_DERSLER = "Güz Dönemi Dersler"
    BAHAR_DONEMI_DERSLER = "Bahar Dönemi Dersler"
    YAZ_DONEMI_DERSLER = "Yaz Dönemi Dersler"
    ISCI_BAYRAMI = "1 Mayıs İşçi Bayrami"
    ULUSAL_EGEMENLIK_VE_COCUK_BAYRAMI = "23 Nisan Ulusal Egemenlik ve Çocuk Bayramı"
    GENCLIK_VE_SPOR_BAYRAMI = "19 Mayıs Genclik ve Spor Bayramı"



# -*-  coding: utf-8 -*-

from datetime import timedelta, date, datetime

from ulakbus.models.form import Form, FormData
from ulakbus.models.hitap.hitap import HizmetKayitlari, HizmetBirlestirme
from ulakbus.models.personel import Personel, Izin
from zengine import forms
from zengine.forms import fields
from zengine.views.crud import CrudView
from zengine.lib.translation import gettext as _, gettext_lazy
import time

class IzinIslemleri(CrudView):
    class Meta:
        # CrudViev icin kullanilacak temel Model
        model = 'Izin'

        # ozel bir eylem listesi hazirlayacagiz. bu sebeple listeyi bosaltiyoruz.
        # kayit tipine bagli olarak ekleyecegimiz eylemleri .append() ile ekleyecegiz
        # object_actions = [
        #     # {'fields': [0, ], 'cmd': 'show', 'mode': 'normal', 'show_as': 'link'},
        # ]

    def goster(self):
        yil = date.today().year
        self.list()
        personel = Personel.objects.get(self.input['id'])
        izin_gun = self.hizmet_izin_yil_hesapla(personel)
        kalan_izin = self.kalan_izin_hesapla()
        bu_yil_kalan_izin = getattr(kalan_izin['yillik'], str(yil), 0)
        gecen_yil_kalan_izin = getattr(kalan_izin['yillik'], str(yil - 1), 0)
        kalan_mazeret_izin = getattr(kalan_izin['mazeret'], str(yil), 0)

        # Personelin izin gün sayısı 365 günden azsa eklemeyi kaldır.
        # Personel pasifse eklemeyi kaldır.
        yil = date.today().year
        izin_hakki_yok_sartlari = [
            izin_gun < 365,
            self.personel_aktif,
            bu_yil_kalan_izin <= 0 and gecen_yil_kalan_izin <= 0 and kalan_mazeret_izin <= 0
        ]
        if any(izin_hakki_yok_sartlari):
            self.ListForm.add = None

        # TODO: Personel bilgileri tabloda gösterilecek
        self.output['object'] = {
            "type": "table",
            "fields": [
                {
                    "name": personel.ad,
                    "surname": personel.soyad
                },
                {
                    "gun": izin_gun,
                    "durum": _(u"Aktif") if self.personel_aktif else _(u"Pasif")
                }, {
                    "izinler": kalan_izin,
                    "hizmet_sure": izin_gun
                }
            ]
        }

    def emekli_sandigi_hesapla(self, personel):
        personel_hizmetler = HizmetKayitlari.objects.filter(tckn=personel.tckn)

        baslangic_liste = set()
        bitis_liste = set()

        for hizmet in personel_hizmetler:
            baslangic_liste.add(hizmet.baslama_tarihi)
            bitis_liste.add(hizmet.bitis_tarihi)

        # clean sets any of blank dates
        blank_dates = [date(1900, 1, 1), '']
        for d in blank_dates:
            if d in baslangic_liste:
                baslangic_liste.remove(d)
            if d in bitis_liste:
                bitis_liste.remove(d)

        # sort dates
        baslangic_liste = sorted(baslangic_liste)
        bitis_liste = sorted(bitis_liste)

        if len(baslangic_liste) > 0:
            self.ilk_izin_hakedis = baslangic_liste[0] + timedelta(365)
        else:
            self.ilk_izin_hakedis = date.today()

        toplam_sure = timedelta(0)
        son_baslama = None

        for baslangic in baslangic_liste:
            if son_baslama is None:
                son_baslama = baslangic
            elif len(bitis_liste) > 0 and baslangic >= bitis_liste[0]:
                toplam_sure += bitis_liste[0] - son_baslama
                son_baslama = baslangic
                bitis_liste.remove(bitis_liste[0])

        if son_baslama is None:
            personel_aktif = False
        elif len(bitis_liste) > 0:
            personel_aktif = False
            toplam_sure += bitis_liste[0] - son_baslama
        else:
            personel_aktif = True
            toplam_sure += date.today() - son_baslama

        # print str(int(toplam_sure.days / 360)) + " Yıl, " + str(int(toplam_sure.days % 360)) + " Gün"
        return toplam_sure.days, personel_aktif

    @staticmethod
    def sgk_hesapla(personel):
        personel_birlestirmeler = HizmetBirlestirme.objects.filter(tckn=personel.tckn)
        toplam_gun = 0

        for hizmet in personel_birlestirmeler:
            toplam_gun += hizmet.sure

        return toplam_gun

    def hizmet_izin_yil_hesapla(self, personel):
        emekli_sandigi_gun, aktif = self.emekli_sandigi_hesapla(personel)
        sgk_gun = self.sgk_hesapla(personel)
        self.personel_aktif = aktif
        return emekli_sandigi_gun + sgk_gun

    def kalan_izin_hesapla(self):
        query = self._apply_list_queries(self.object.objects.filter())
        yillik_izinler = dict()
        mazeret_izinler = dict()

        if self.ilk_izin_hakedis > date.today():
            return {'yillik': None, 'mazeret': None}

        for yil in range(self.ilk_izin_hakedis.year, date.today().year + 1):
            yillik_izinler[yil] = 20
            mazeret_izinler[yil] = 10

        for izin in query:
            if izin.tip == 1:
                yil = izin.baslangic.year - 1
                if yil in yillik_izinler.keys() and yillik_izinler[yil] > 0:
                    yillik_izinler[yil] -= (izin.bitis - izin.baslangic).days
                else:
                    yil += 1
                    yillik_izinler[yil] -= (izin.bitis - izin.baslangic).days
            elif izin.tip == 5:
                mazeret_izinler[yil] -= (izin.bitis - izin.baslangic).days

        return {'yillik': yillik_izinler, 'mazeret': mazeret_izinler}


class IzinBasvuru(CrudView):
    """İzin Başvuru İş Akışı

    İzin Başvuru iş akışı dört adımdan ve iki yoldan oluşur.

    İlk yol iki iş akışı adımından oluşur.

    İzin Başvuru Formu Göster:

    İzin başvurusu yaopacak personel için izin başvuru gösterilir.
    Personelin geçen yıl kullandığı izin sayısı, geçen yıldan kalan
    izin sayısı, bu yıl kullandığı izin sayısı, bu yıl kalan izin sayısı
    ve toplamda kalan izin sayısı gösterilir.

    İzin Başvuru Kaydet:

    Personelin izin başvurusu kaydedilir.

    İkinci yol iki iş akışı adımından 2 adımdan oluşur.

    İzin Başvuru Göster:

    İzin işlemleriyle ilgilenen personel, izin başvurusu yapan personelin izin
    bilgilerini görüntüler.

    İzin Başvuru Sonuç Kaydet:

    İzin işlemleriyle ilgilenen personel, izin başvurusu yapan personelin
    iznini kaydeder.

    """
    class Meta:
        model = 'Izin'

    class IzinBasvuruForm(forms.JsonForm):

        """
        İzin Başvuru iş akışı için kullanılacak formdur.

        """

        izin_turleri = ((1, gettext_lazy("Yıllık İzin")),
                        (2, gettext_lazy("Mazeret İzni")),
                        (3, gettext_lazy("Refakat İzni")),
                        (4, gettext_lazy("Fazla Mesai İzni")),
                        (5, gettext_lazy("Ücretsiz İzin")),
        )
        gecerli_yil = date.today().year
        yillar = ((gecerli_yil, gecerli_yil),)

        izin_turu = fields.Integer(gettext_lazy(u"İzin Türü Seçiniz"), choices=izin_turleri)
        izin_ait_yil = fields.Integer(gettext_lazy(u"Ait Olduğu Yıl"), choices=yillar)
        izin_baslangic = fields.Date(gettext_lazy(u"İzin Başlangıç Tarihi"))
        izin_bitis = fields.Date(gettext_lazy(u"İzin Bitiş Tarihi"))
        izin_adres = fields.Text(gettext_lazy(u"İzindeki Adres"))
        personel_ad_soyad = fields.String(gettext_lazy(u"İzin Talep Eden"))
        personel_gorev = fields.String(gettext_lazy(u"Görevi"))
        personel_sicil_no = fields.String(gettext_lazy(u"Sicil no"))
        personel_birim = fields.String(gettext_lazy(u"Birimi"))
        yol_izni = fields.Boolean(gettext_lazy(u"Yol İzni"))
        toplam_izin_gun = fields.Integer(gettext_lazy(u"Kullanacağı İzin Süresi(Gün)"))
        toplam_kalan_izin = fields.Integer(gettext_lazy(u"Kalan İzin Süresi(Gün)"))

        ileri = fields.Button(gettext_lazy(u"İleri"))

    TOPLAM_IZIN_SAYISI = 10

    def izin_kontrol(self):
        self.current.task_data['sicil_no'] = self.current.user.personel.kurum_sicil_no
        lst_form_data = FormData.objects.filter(user_id=self.current.user.key)
        guncel_yil = date.today().year
        onceki_yil = guncel_yil - 1
        onceki_yil_izin_bilgileri = IzinBasvuru.onceki_yil_izinlerini_bul(lst_form_data, onceki_yil)
        guncel_yil_izin_bilgileri = IzinBasvuru.guncel_izinleri_bul(lst_form_data, guncel_yil,
                                                                    onceki_yil_izin_bilgileri[1])
        if guncel_yil_izin_bilgileri[1] <= 0:
            self.current.output['msgbox'] = {
                'type': 'info', "title": _(u'İzin Başvuru'),
                "msg": _(u'%s yılı için izin kullanımlarınız bitmiştir.') % guncel_yil}
            self.current.task_data['cmd'] = 'end'
        else:
            self.current.task_data['cmd'] = 'izin_basvuru_formu_goster'

    def izin_basvuru_formu_goster(self):
        """
        İzin başvurusu yaopacak personel için izin başvuru gösterilir.
        Personelin geçen yıl kullandığı izin sayısı, geçen yıldan kalan
        izin sayısı, bu yıl kullandığı izin sayısı, bu yıl kalan izin sayısı
        ve toplamda kalan izin sayısı gösterilir.

        """

        personel_id = self.current.user.personel.key
        self.current.task_data['personel_id'] = personel_id
        guncel_yil = date.today().year
        onceki_yil = guncel_yil - 1
        lst_form_data = FormData.objects.filter(user_id=self.current.user.key)
        onceki_yil_izin = IzinBasvuru.onceki_yil_izinlerini_bul(lst_form_data, onceki_yil)
        guncel_yil_izin = IzinBasvuru.guncel_izinleri_bul(lst_form_data, guncel_yil, onceki_yil_izin[1])

        _form = self.IzinBasvuruForm(current=self.current,
                                     title=_(u"İzin Talep Formu"),
                                     exclude=['personel_ad_soyad', 'personel_gorev',
                                              'toplam_kalan_izin', 'toplam_izin_gun',
                                              'yol_izni', 'personel_birim',
                                              'personel_sicil_no'])

        _form.help_text = _(u"""{onceki} yılına ait izinli gün sayınız {onceki_izin}, Kalan gün sayınız {onceki_kalan}
                          {guncel} yılına ait izinli gün sayınız {guncel_izin},Toplam izinli gün sayınız  {guncel_kalan}
                          'dir.
                          """).format(onceki=onceki_yil, onceki_izin=onceki_yil_izin[0], onceki_kalan=onceki_yil_izin[1],
                                      guncel=guncel_yil, guncel_izin=guncel_yil_izin[0], guncel_kalan=guncel_yil_izin[1],
                           )
        self.form_out(_form)

    @staticmethod
    def guncel_izinleri_bul(lst_form_data, guncel_yil, onceki_yil_kalan_izin):
        izinli_gun = 0
        for item in lst_form_data:
            date_format = '%d.%m.%Y'
            a = datetime.strptime(item.data['izin_baslangic'], date_format)
            b = datetime.strptime(item.data['izin_bitis'], date_format)
            if item.data['izin_ait_yil'] == guncel_yil:
                delta = b - a
                izinli_gun += delta.days

        return izinli_gun, (IzinBasvuru.TOPLAM_IZIN_SAYISI - izinli_gun) + onceki_yil_kalan_izin

    @staticmethod
    def onceki_yil_izinlerini_bul(lst_form_data, onceki_yil):
        izinli_gun = 0
        for item in lst_form_data:
            date_format = '%d.%m.%Y'
            # "%Y-%m-%dT%H:%M:%S.000Z"
            if item.data['izin_ait_yil'] == onceki_yil:
                a = datetime.strptime(item.data['izin_baslangic'], date_format)
                b = datetime.strptime(item.data['izin_bitis'], date_format)
                delta = b - a
                izinli_gun += delta.days

        return izinli_gun, IzinBasvuru.TOPLAM_IZIN_SAYISI - izinli_gun

    def izin_basvuru_kaydet(self):
        """
        Personelin izin başvurusu kaydedilir.

        """

        izin_form_data = self.input['form']
        # gereksiz form alanlarını sil
        if 'ileri' in izin_form_data:
            del izin_form_data['ileri']

        self.current.task_data['izin_form_data'] = izin_form_data
        form_personel = Personel.objects.get(self.current.task_data['personel_id'])
        izin_form = Form.objects.get(ad="İzin Formu")
        form_data = FormData()
        form_data.form = izin_form
        form_data.user = form_personel.user
        form_data.data = izin_form_data
        form_data.date = date.today()
        form_data.blocking_save()
        time.sleep(1)
        self.current.task_data['izin_form_data_key'] = form_data.key

        msg = {"title": _(u'İzin Başvurusu Yapıldı'),
               "body": _(u'%s %s tarih aralığı için yaptığınız izin talebi başarıyla alınmıştır.') % (
                   self.input['form']['izin_baslangic'], self.input['form']['izin_bitis'])}
        # workflowun bu kullanıcı için bitişinde verilen mesajı ekrana bastırır
        self.current.task_data['LANE_CHANGE_MSG'] = msg

    def izin_basvuru_sonuc_kaydet(self):

        """
        İzin işlemleriyle ilgilenen personel, izin başvurusu yapan personelin
        iznini kaydeder.
        TODO : Personele vekalet edecek kişinin seçimi yapılmalı

        """
        from datetime import date
        personel = Personel.objects.get(self.current.task_data['izin_personel_key'])
        form = self.input['form']
        izin = Izin()
        izin.tip = form['izin_turu']
        izin.baslangic = form['izin_baslangic']
        izin.bitis = form['izin_bitis']
        izin.onay = date.today()
        izin.adres = form['izin_adres']
        izin.personel = personel
        izin.save()
        self.current.output['msgbox'] = {
            'type': 'info', "title": _(u'İzin Başvurusu Onaylandı'),
            "msg": _(u'İzin talebi başarıyla onaylanmıştır.')
        }

    def izin_basvuru_goster(self):
        """
        İzin işlemleriyle ilgilenen personel, izin başvurusu yapan personelin izin
        bilgilerini görüntüler.

        """
        form_data = FormData.objects.get(self.current.task_data['izin_form_data_key'])
        _form = self.IzinBasvuruForm(current=self.current,
                                     title=_(u"İzin Talep Önizleme Formu"),
                                     )
        basvuru_data = form_data.data
        personel = form_data.user.personel
        self.current.task_data['izin_personel_key'] = personel.key
        if personel.atama.exist:
            _form.personel_gorev = (personel.atama.kadro.get_unvan_display()
                                    if personel.atama.kadro.exist else None)
            _form.personel_sicil_no = personel.atama.kurum_sicil_no
        if personel.birim.exist:
            _form.personel_birim = personel.birim.name

        date_format = "%d.%m.%Y"
        a = datetime.strptime(basvuru_data['izin_baslangic'], date_format)
        b = datetime.strptime(basvuru_data['izin_bitis'], date_format)
        delta = b - a
        _form.izin_turu = basvuru_data['izin_turu']
        _form.izin_ait_yil = basvuru_data['izin_ait_yil']
        _form.izin_baslangic = basvuru_data['izin_baslangic']
        _form.izin_bitis = basvuru_data['izin_bitis']
        _form.izin_adres = basvuru_data['izin_adres']
        _form.personel_sicil_no = self.current.task_data['sicil_no']
        _form.toplam_izin_gun = delta.days
        _form.toplam_kalan_izin = IzinBasvuru.TOPLAM_IZIN_SAYISI - delta.days
        _form.personel_ad_soyad = "%s %s" % (personel.ad, personel.soyad)
        self.form_out(_form)



# -*-  coding: utf-8 -*-

# Copyright (C) 2015 ZetaOps Inc.
#
# This file is licensed under the GNU General Public License v3
# (GPLv3).  See LICENSE.txt for details.

from pyoko import Model, field
from zengine.lib.translation import gettext_lazy as __


class ZatoServiceFile(Model):

    cluster_id = field.Integer(__(u'Zato cluster'))
    service_payload_name = field.String(__(u"Module’s file name"))
    service_payload = field.String(__(u"BASE64-encoded"))
    deploy = field.Boolean(__(u"Deploy Service"), default=False)

    def __unicode__(self):
        return self.service_payload_name


class ZatoServiceChannel(Model):
    cluster_id = field.Integer(__(u'Zato cluster'))
    channel_id = field.Integer(__(u"Channel id"))
    channel_name = field.String(__(u"Channel name"))
    channel_connection = field.String(__(u"channel or outgoing connection"))
    channel_transport = field.String(__(u"plain_http or soap"))
    channel_url_path = field.String(__(u"Example: /test/zato-url-path"))
    channel_data_format = field.String(__(u"xml or json"))
    channel_is_internal = field.Boolean(default=False)
    channel_is_active = field.Boolean(default=True)
    service_id = field.Integer(__(u"Service Id"))
    service_name = field.String(__(u"Service name"))
    deploy = field.Boolean(__(u"Deploy Channel"), default=False)

    def __unicode__(self):
        return self.channel_name



# -*-  coding: utf-8 -*-

# Copyright (C) 2015 ZetaOps Inc.
#
# This file is licensed under the GNU General Public License v3
# (GPLv3).  See LICENSE.txt for details.

"""Banka Borç Sorgulama

Banka tarafından öğrencinin borçlarını sorgulamak için kullanılan Zato Servisi.

Yetkilendirme için banka bilgileri,
öğrenci borç bilgilerine erişim için öğrenci numarası verilir.

Elde edilen bilgilerle öğrencinin tüm borç bilgileri (mevcutsa) listelenir.

Example:

    Servise JSON nesnesi kullanılarak istek gönderilmesi:

    .. code-block:: json

        $ curl http://localhost:11223/banka-borc-getir -d '{
            "banka_kodu": "kod",
            "bank_username": "user",
            "bank_password": "pass",
            "ogrenci_no":"1234567890",
            "sube_kodu":"sube",
            "kanal_kodu":"kanal",
            "mesaj_no":"mesaj"
            }'


    İsteğe dönen cevap:

    .. code-block:: json

        $ {"borc_response": [{
            "tahakkuk_referans_no": "tahakkuk",
            "ucret_turu": 1,
            "ad_soyad": "Ad Soyad",
            "bank_password": "pass",
            "bank_username": "user",
            "mesaj_no": "mesaj",
            "sube_kodu": "sube",
            "hata_mesaj": null,
            "borc_ack": "aciklama",
            "son_odeme_tarihi": "07122015",
            "ogrenci_no": "1234567890",
            "banka_kodu": "kod",
            "borc": 214.0,
            "kanal_kodu": "kanal",
            "mesaj_statusu": "K"
            },
            ...
        ]}


    Servise XML kullanılarak istek gönderilmesi:

    .. code-block:: xml

        $ curl http://localhost:11223/banka-borc.banka-borc-getir
            -H "SOAPAction:banka-borc.banka-borc-getir"
            -d '<soapenv:Envelope xmlns:soapenv="http://schemas.xmlsoap.org/soap/envelope/"
                xmlns:zato="https://zato.io/ns/20130518">
                    <soapenv:Body>
                        <zato:borc_request>
                            <zato:banka_kodu>kod</zato:banka_kodu>
                            <zato:bank_username>user</zato:bank_username>
                            <zato:bank_password>pass</zato:bank_password>
                            <zato:ogrenci_no>1234567890</zato:ogrenci_no>
                            <zato:sube_kodu>sube</zato:sube_kodu>
                            <zato:kanal_kodu>kanal</zato:kanal_kodu>
                            <zato:mesaj_no>mesaj</zato:mesaj_no>
                        </zato:borc_request>
                    </soapenv:Body>
                </soapenv:Envelope>'


    İsteğe dönen cevap:

    .. code-block:: xml

        $ <?xml version='1.0' encoding='UTF-8'?>
            <soap:Envelope xmlns:soap="http://schemas.xmlsoap.org/soap/envelope/"
            xmlns="https://zato.io/ns/20130518">
            <soap:Body>
                <borc_response>
                    <zato_env>
                        <cid>K068TA34MK4E0KPWMW1EK5HS9TB8</cid>
                        <result>ZATO_OK</result>
                    </zato_env>
                    <item_list xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance">
                        <item>
                            <mesaj_statusu>K</mesaj_statusu>
                            <hata_mesaj xsi:nil="true"/>
                            <banka_kodu>kod</banka_kodu>
                            <sube_kodu>sube</sube_kodu>
                            <kanal_kodu>kanal</kanal_kodu>
                            <mesaj_no>mesaj</mesaj_no>
                            <bank_username>user</bank_username>
                            <bank_password>pass</bank_password>
                            <ogrenci_no>1234567890</ogrenci_no>
                            <ad_soyad>Ad Soyad</ad_soyad>
                            <ucret_turu>1</ucret_turu>
                            <tahakkuk_referans_no>tahakkuk</tahakkuk_referans_no>
                            <son_odeme_tarihi>07122015</son_odeme_tarihi>
                            <borc>214.0</borc>
                            <borc_ack>aciklama</borc_ack>
                        </item>
                        <item>
                            ...
                        </item>
                        ...
                    </item_list>
                </borc_response>
            </soap:Body>
        </soap:Envelope>

"""

from ulakbus.services.common.banka import BankaService
from ulakbus.models.ogrenci import OgrenciProgram, Borc
from pyoko.exceptions import ObjectDoesNotExist
import json
from datetime import date


class BankaBorcGetir(BankaService):
    """
    Banka servisinden kalıtılarak gerçekleştirilen Banka Borç Sorgulama servisi.

    """

    HAS_CHANNEL = True

    def __init__(self):
        super(BankaBorcGetir, self).__init__()

    class SimpleIO():
        """
        Servis girdilerinin ve çıktılarının belirlendiği yapı.

        Note:
            SimpleIO sınıfında belirlenen özellikler,
            doğrudan yer aldığı servisin özellikleri olmaktadır.

            - borc_request (str): Servise gelen isteğin (JSON, XML) ismi
            - borc_response (str): Servisten dönen veriyi içeren (payload)
              cevabın (JSON, XML) ismi
            - banka_kodu (str): Üniversite tarafından bankaya verilen kod
            - bank_username (str): Üniversite tarafından bankaya verilen kullanıcı adı
            - bank_password (str): Üniversite tarafından bankaya verilen şifre
            - sube_kodu (str): Bankaların şubeleri için hali hazırda kullandıkları kodlar
            - kanal_kodu (str): G (Gişe), İ (İnternet), A (ATM), T (AloBanka) vb.
            - mesaj_no (str): Banka tarafından üretilen kod.
            - ogrenci_no (str): Borçları sorgulanan öğrencinin numarası
            - mesaj_statusu (str): K (Kabul), R (Ret)
            - hata_mesaji (str): Hata mesajı içeriği veya null
            - ad_soyad (str): Öğrenci adı soyadı
            - ucret_turu (int): Borcun ne için ödeneceği
            - tahakkuk_referans_no (str): Her tahakkuka verilen referans numarası
            - son_odeme_tarihi (str): Borcun son ödeme tarihi (DDMMYY formatında)
            - borc (float): Borcun miktarı
            - borc_ack (str): Borç açıklaması

        """

        request_elem = 'borc_request'
        response_elem = 'borc_response'
        input_required = ('banka_kodu', 'sube_kodu', 'kanal_kodu', 'mesaj_no', 'bank_username',
                          'bank_password', 'ogrenci_no')
        output_required = ('mesaj_statusu','hata_mesaj')
        output_optional = ('banka_kodu', 'sube_kodu', 'kanal_kodu', 'mesaj_no', 'bank_username',
                           'bank_password', 'ogrenci_no', 'ad_soyad', 'ucret_turu',
                           'tahakkuk_referans_no', 'son_odeme_tarihi', 'borc', 'borc_ack')

    def handle(self):
        """
        Servis çağrıldığında tetiklenen metod.

        Yetkilendirme işleminin sonucuna göre servis çalışmaya devam eder.

        """

        super(BankaBorcGetir, self).handle()

    def get_data(self):
        """
        Öğrencinin bilgilerine göre tüm borç bilgilerini döndürür.

        Öğrenci numarasına göre, öğrencinin dahil olduğu programdan bilgileri alınır ve
        öğrenciye ait (varsa) borçlara erişilir. Borçlar bir liste halinde geriye döndürülür.

        Öğrenci sistemde bulunamadıysa veya beklenmeyen bir hatayla karşılaşıldıysa
        cevap olarak hata mesajı döndürülür.

        Raises:
            ObjectDoesNotExist: Öğrenci numarası sistemde kayıtlı değildir.
            Exception: Öğrencinin borçları sorgulanırken hatayla karşılaşılmıştır.

        """

        super(BankaBorcGetir, self).get_data()

        inp = self.request.input

        try:
            ogrenci = OgrenciProgram.objects.get(ogrenci_no=inp.ogrenci_no).ogrenci

            borclar = Borc.objects.filter(ogrenci=ogrenci)
            for borc in borclar:
                borc_response = {
                    'banka_kodu': inp.banka_kodu,
                    'sube_kodu': inp.sube_kodu,
                    'kanal_kodu': inp.kanal_kodu,
                    'mesaj_no': inp.mesaj_no,
                    'bank_username': inp.bank_username,
                    'bank_password': inp.bank_password,
                    'ogrenci_no': inp.ogrenci_no,
                    'ad_soyad': ogrenci.ad + " " + ogrenci.soyad,
                    'ucret_turu': borc.sebep,
                    'tahakkuk_referans_no': borc.tahakkuk_referans_no,
                    'son_odeme_tarihi': date.strftime(borc.son_odeme_tarihi, format='%d%m%Y'),
                    'borc': borc.miktar,
                    'borc_ack': borc.aciklama,
                    'mesaj_statusu': 'K',
                    'hata_mesaj': None
                }

                self.logger.info("Borc bilgisi: %s" % json.dumps(borc_response))
                self.response.payload.append(borc_response)

        except ObjectDoesNotExist:
            self.logger.exception("Ogrenci numarasi bulunamadi.")
            self.response.payload['mesaj_statusu'] = "R"
            self.response.payload['hata_mesaj'] = "Ogrenci numarasi bulunamadi!"
        except Exception:
            self.logger.exception("Borc sorgulama sirasinda hata olustu.")
            self.response.payload['mesaj_statusu'] = "R"
            self.response.payload['hata_mesaj'] = "Borc sorgulama hatasi!"




# -*-  coding: utf-8 -*-
# Copyright (C) 2015 ZetaOps Inc.
#
# This file is licensed under the GNU General Public License v3
# (GPLv3).  See LICENSE.txt for details.
""" Personel Modülü

Bu modül Ulakbüs uygulaması için personel modelini ve  personel ile ilişkili modelleri içerir.

"""
from .hitap.hitap_sebep import HitapSebep
from pyoko.lib.utils import lazy_property

from pyoko import Model, field
from zengine.lib.translation import gettext_lazy as _, gettext
from ulakbus.lib.personel import gorunen_kademe_hesapla
from .auth import Unit, User
from ulakbus.settings import SICIL_PREFIX
from .auth import AbstractRole


class Personel(Model):
    """Personel Modeli

    Personelin özlük ve iletişim bilgilerini içerir.

    """

    tckn = field.String(_(u"TC No"))
    kurum_sicil_no_int = field.Integer(_(u"Kurum Sicil No"))
    ad = field.String(_(u"Adı"))
    soyad = field.String(_(u"Soyadı"))
    cinsiyet = field.Integer(_(u"Cinsiyet"), choices='cinsiyet')
    uyruk = field.String(_(u"Uyruk"))
    ikamet_adresi = field.String(_(u"İkamet Adresi"))
    ikamet_il = field.String(_(u"İkamet İl"))
    ikamet_ilce = field.String(_(u"İkamet İlçe"))
    adres_2 = field.String(_(u"Adres 2"))
    oda_no = field.String(_(u"Oda Numarası"))
    oda_tel_no = field.String(_(u"Oda Telefon Numarası"))
    cep_telefonu = field.String(_(u"Cep Telefonu"))
    e_posta = field.String(_(u"E-Posta"))
    e_posta_2 = field.String(_(u"E-Posta 2"))
    e_posta_3 = field.String(_(u"E-Posta 3"))
    web_sitesi = field.String(_(u"Web Sitesi"))
    yayinlar = field.String(_(u"Yayınlar"))
    projeler = field.String(_(u"Projeler"))
    kan_grubu = field.String(_(u"Kan Grubu"))
    ehliyet = field.String(_(u"Ehliyet"))
    verdigi_dersler = field.String(_(u"Verdiği Dersler"))
    biyografi = field.Text(_(u"Biyografi"))
    notlar = field.Text(_(u"Notlar"), required=False)
    engelli_durumu = field.String(_(u"Engellilik"))
    engel_grubu = field.String(_(u"Engel Grubu"))
    engel_derecesi = field.String(_(u"Engel Derecesi"))
    engel_orani = field.Integer(_(u"Engellilik Oranı"))
    cuzdan_seri = field.String(_(u"Seri"))
    cuzdan_seri_no = field.String(_(u"Seri No"))
    baba_adi = field.String(_(u"Ana Adı"))
    ana_adi = field.String(_(u"Baba Adı"))
    dogum_tarihi = field.Date(_(u"Doğum Tarihi"), format="%d.%m.%Y")
    dogum_yeri = field.String(_(u"Doğum Yeri"))
    medeni_hali = field.Integer(_(u"Medeni Hali"), choices="medeni_hali")
    kayitli_oldugu_il = field.String(_(u"İl"))
    kayitli_oldugu_ilce = field.String(_(u"İlçe"))
    kayitli_oldugu_mahalle_koy = field.String(_(u"Mahalle/Köy"))
    kayitli_oldugu_cilt_no = field.String(_(u"Cilt No"))
    kayitli_oldugu_aile_sira_no = field.String(_(u"Aile Sıra No"))
    kayitli_oldugu_sira_no = field.String(_(u"Sıra No"))
    kimlik_cuzdani_verildigi_yer = field.String(_(u"Cüzdanın Verildiği Yer"))
    kimlik_cuzdani_verilis_nedeni = field.String(_(u"Cüzdanın Veriliş Nedeni"))
    kimlik_cuzdani_kayit_no = field.String(_(u"Cüzdan Kayıt No"))
    kimlik_cuzdani_verilis_tarihi = field.String(_(u"Cüzdan Kayıt Tarihi"))

    kazanilmis_hak_derece = field.Integer(_(u"Güncel Kazanılmış Hak Derece"), index=True)
    kazanilmis_hak_kademe = field.Integer(_(u"Güncel Kazanılmış Hak Kademe"), index=True)
    kazanilmis_hak_ekgosterge = field.Integer(_(u"Kazanılmış Hak Ek Gösterge"), index=True)

    gorev_ayligi_derece = field.Integer(_(u"Güncel Görev Aylığı Derece"), index=True)
    gorev_ayligi_kademe = field.Integer(_(u"Güncel Görev Aylığı Kademe"), index=True)
    gorev_ayligi_ekgosterge = field.Integer(_(u"Görev Aylığı Ek Gösterge"), index=True)

    emekli_muktesebat_derece = field.Integer(_(u"Güncel Emekli Müktesebat Derece"), index=True)
    emekli_muktesebat_kademe = field.Integer(_(u"Güncel Emekli Müktesebat Kademe"), index=True)
    emekli_muktesebat_ekgosterge = field.Integer(_(u"Emekli Müktesebat Ek Gösterge"), index=True)

    kh_sonraki_terfi_tarihi = field.Date(_(u"Kazanılmış Hak Sonraki Terfi Tarihi"), index=True,
                                         format="%d.%m.%Y")
    ga_sonraki_terfi_tarihi = field.Date(_(u"Görev Aylığı Sonraki Terfi Tarihi"), index=True,
                                         format="%d.%m.%Y")
    em_sonraki_terfi_tarihi = field.Date(_(u"Emekli Müktesebat Sonraki Terfi Tarihi"), index=True,
                                         format="%d.%m.%Y")

    birim = Unit(_(u"Birim"))

    # Personelin Kendi Ünvanı
    unvan = field.Integer(_(u"Personel Unvan"), index=True, choices="unvan_kod", required=False)

    # Aşağıdaki bilgiler atama öncesi kontrol edilecek, Doldurulması istenecek
    emekli_sicil_no = field.String(_(u"Emekli Sicil No"), index=True)
    emekli_giris_tarihi = field.Date(_(u"Emekliliğe Giriş Tarihi"), index=True, format="%d.%m.%Y")

    personel_turu = field.Integer(_(u"Personel Türü"), choices="personel_turu")
    hizmet_sinifi = field.Integer(_(u"Hizmet Sınıfı"), choices="hizmet_sinifi")
    statu = field.Integer(_(u"Statü"), choices="personel_statu")
    brans = field.String(_(u"Branş"), index=True)

    # akademik personeller icin sozlesme sureleri
    gorev_suresi_baslama = field.Date(_(u"Görev Süresi Başlama"), index=True, format="%d.%m.%Y")
    gorev_suresi_bitis = field.Date(_(u"Görev Süresi Bitiş"), index=True, format="%d.%m.%Y")

    # todo: durum_degisikligi yonetimi
    # kurumda ilk goreve baslama bilgileri, atama modelinden elde edilip
    # edilemeyecegini soracagiz. mevcut otomasyonda ayrilmalar da burada tutuluyor.
    # bunu tarih ve durum_degisikligi fieldlarindan olusan bir listnode seklinde tutabiliriz.
    goreve_baslama_tarihi = field.Date(_(u"Göreve Başlama Tarihi"), index=True, format="%d.%m.%Y")
    baslama_sebep = HitapSebep()

    # aday ve idari memurlar icin mecburi hizmet suresi
    mecburi_hizmet_suresi = field.Date(_(u"Mecburi Hizmet Süresi"), index=True, format="%d.%m.%Y")

    # Arama için kullanılacak Flaglar
    kadro_derece = field.Integer(default=0)
    aday_memur = field.Boolean()
    arsiv = field.Boolean()  # ayrilmis personeller icin gecerlidir.

    user = User(one_to_one=True)

    class Meta:
        app = 'Personel'
        verbose_name = _(u"Personel")
        verbose_name_plural = _(u"Personeller")
        list_fields = ['ad', 'soyad', 'tckn', 'durum']
        search_fields = ['ad', 'soyad', 'cep_telefonu', 'tckn']

    def durum(self):
        return self.nufus_kayitlari.durum if self.nufus_kayitlari.key else None

    durum.title = "Durum"

    @lazy_property
    def gorunen_kazanilmis_hak_kademe(self):
        return gorunen_kademe_hesapla(int(self.kazanilmis_hak_derece),
                                      int(self.kazanilmis_hak_kademe))

    @lazy_property
    def gorunen_gorev_ayligi_kademe(self):
        return gorunen_kademe_hesapla(int(self.gorev_ayligi_derece), int(self.gorev_ayligi_kademe))

    @lazy_property
    def gorunen_emekli_muktesebat_kademe(self):
        return gorunen_kademe_hesapla(int(self.emekli_muktesebat_derece),
                                      int(self.emekli_muktesebat_kademe))

    @lazy_property
    def atama(self):
        """atama

        Personelin atama bilgilerini iceren atama nesnesini getirir.

        Returns:
            Atama örneği (instance)

        """
        # Mevcut pyoko API'i ile uyumlu olmasi icin, geriye bos bir Atama nesnesi dondurur.
        atamalar = Atama.objects.set_params(sort='goreve_baslama_tarihi desc').filter(personel=self)
        return atamalar[0] if atamalar else Atama()

    @lazy_property
    def kadro(self):
        """Kadro

        Personelin atama bilgilerinden kadrosuna erişir.

        Returns:
            Kadro örneği (instance)

        """

        return self.atama.kadro

    @lazy_property
    def sicil_no(self):
        """Kadro

        Personelin atama bilgilerinden sicil numarasina erişir.

        Returns:
            Sicil No (str)

        """

        return self.atama.kurum_sicil_no

    @property
    def kurum_sicil_no(self):
        return "%s-%s" % (SICIL_PREFIX, self.kurum_sicil_no_int)

    def __unicode__(self):
        return gettext(u"%(ad)s %(soyad)s") % {'ad': self.ad, 'soyad': self.soyad}


class AdresBilgileri(Model):
    """Adres Bilgileri Modeli

    Personele ait adres bilgilerini içeren modeldir.

    Personelin birden fazla adresi olabilir.

    """

    ad = field.String(_(u"Adres Adı"))
    adres = field.String(_(u"Adres"))
    ilce = field.String(_(u"İlçe"))
    il = field.String(_(u"İl"))
    personel = Personel()

    class Meta:
        verbose_name = _(u"Adres Bilgisi")
        verbose_name_plural = _(u"Adres Bilgileri")

    def __unicode__(self):
        return "%s %s" % (self.ad, self.il)


class KurumIciGorevlendirmeBilgileri(Model):
    """Kurum İçi Görevlendirme Bilgileri Modeli

    Personelin, kurum içi görevlendirme bilgilerine ait modeldir.

    Görevlendirme bir birim ile ilişkili olmalıdır.

    """

    gorev_tipi = field.String(_(u"Görev Tipi"), choices="gorev_tipi")
    kurum_ici_gorev_baslama_tarihi = field.Date(_(u"Başlama Tarihi"), format="%d.%m.%Y")
    kurum_ici_gorev_bitis_tarihi = field.Date(_(u"Bitiş Tarihi"), format="%d.%m.%Y")
    birim = Unit()
    soyut_rol = AbstractRole()
    aciklama = field.String(_(u"Açıklama"))
    resmi_yazi_sayi = field.String(_(u"Resmi Yazı Sayı"))
    resmi_yazi_tarih = field.Date(_(u"Resmi Yazı Tarihi"), format="%d.%m.%Y")
    personel = Personel()

    class Meta:
        """``form_grouping`` kullanıcı arayüzeyinde formun temel yerleşim planını belirler.

        Layout grid (toplam 12 sütun) içerisindeki değerdir.

        Her bir ``layout`` içinde birden fazla form grubu yer alabilir: ``groups``

        Her bir grup, grup başlığı ``group_title``, form öğeleri ``items`` ve bu grubun
        açılır kapanır olup olmadığını belirten boolen bir değerden ``collapse`` oluşur.

        """

        verbose_name = _(u"Kurum İçi Görevlendirme")
        verbose_name_plural = _(u"Kurum İçi Görevlendirmeler")
        form_grouping = [
            {
                "layout": "4",
                "groups": [
                    {
                        "group_title": _(u"Görev"),
                        "items": ["gorev_tipi", "kurum_ici_gorev_baslama_tarihi",
                                  "kurum_ici_gorev_bitis_tarihi",
                                  "birim", "aciklama"],
                        "collapse": False
                    }
                ]

            },
            {
                "layout": "2",
                "groups": [
                    {
                        "group_title": _(u"Resmi Yazi"),
                        "items": ["resmi_yazi_sayi", "resmi_yazi_tarih"],
                        "collapse": False
                    }
                ]

            },
        ]

    def __unicode__(self):
        return "%s %s" % (self.gorev_tipi, self.aciklama)


class KurumDisiGorevlendirmeBilgileri(Model):
    """Kurum Dışı Görevlendirme Bilgileri Modeli

    Personelin bağlı olduğu kurumun dışındaki görev bilgilerine ait modeldir.

    """

    gorev_tipi = field.Integer(_(u"Görev Tipi"))
    kurum_disi_gorev_baslama_tarihi = field.Date(_(u"Başlama Tarihi"), format="%d.%m.%Y")
    kurum_disi_gorev_bitis_tarihi = field.Date(_(u"Bitiş Tarihi"), format="%d.%m.%Y")
    aciklama = field.Text(_(u"Açıklama"))
    resmi_yazi_sayi = field.String(_(u"Resmi Yazı Sayı"))
    resmi_yazi_tarih = field.Date(_(u"Resmi Yazı Tarihi"), format="%d.%m.%Y")
    maas = field.Boolean(_(u"Maaş"))
    yevmiye = field.Boolean(_(u"Yevmiye"), default=False)
    yolluk = field.Boolean(_(u"Yolluk"), default=False)
    ulke = field.Integer(_(u"Ülke"), default="90", choices="ulke")
    soyut_rol = AbstractRole()
    personel = Personel()

    class Meta:
        verbose_name = _(u"Kurum Dışı Görevlendirme")
        verbose_name_plural = _(u"Kurum Dışı Görevlendirmeler")
        list_fields = ["ulke", "gorev_tipi", "kurum_disi_gorev_baslama_tarihi"]
        list_filters = ["ulke", "gorev_tipi", "kurum_disi_gorev_baslama_tarihi"]
        search_fields = ["aciklama", ]
        form_grouping = [
            {
                "layout": "4",
                "groups": [
                    {
                        "group_title": _(u"Görev"),
                        "items": ["gorev_tipi", "kurum_disi_gorev_baslama_tarihi",
                                  "kurum_disi_gorev_bitis_tarihi",
                                  "ulke",
                                  "aciklama"],
                        "collapse": False
                    }
                ]

            },
            {
                "layout": "2",
                "groups": [
                    {
                        "group_title": _(u"Resmi Yazi"),
                        "items": ["resmi_yazi_sayi", "resmi_yazi_tarih"],
                        "collapse": False
                    },
                    {
                        "items": ["maas", "yevmiye", "yolluk"],
                        "collapse": False
                    }
                ]

            },
        ]

    def __unicode__(self):
        return "%s %s %s" % (self.gorev_tipi, self.aciklama, self.ulke)


class Kadro(Model):
    """Kadro Modeli

    Kurum için ayrılmış Kadro bilgilerine modeldir.

    Kadrolar 4 halde bulunabilirler: SAKLI, IZINLI, DOLU ve BOŞ

        * SAKLI: Saklı kadro, atama yapılmaya müsadesi olmayan, etkinlik onayı alınmamış
          fakat kurum için ayrılmış potansiyel kadroyu tanımlar.
        * IZINLI: Henüz atama yapılmamış, fakat etkinlik onayı alınmış kadroyu tanımlar.
        * DOLU: Bir personel tarafından işgal edilmiş bir kadroyu tanımlar. Ataması yapılmıştır.
        * BOŞ: Çeşitli sebepler ile DOLU iken boşaltılmış kadroyu tanınmlar.

    ``unvan`` ve ``unvan_kod`` karşıt alanlardır. Birisi varken diğeri mevcut olamaz.

    """

    kadro_no = field.Integer(_(u"Kadro No"), required=False)
    derece = field.Integer(_(u"Derece"), required=False)
    durum = field.Integer(_(u"Durum"), choices="kadro_durumlari", required=False)
    birim = Unit(_(u"Birim"), required=False)
    aciklama = field.String(_(u"Açıklama"), index=True, required=False)
    unvan = field.Integer(_(u"Unvan"), index=True, choices="unvan_kod", required=False)
    unvan_aciklama = field.String(_(u"Unvan Aciklama"), index=True, required=False)

    class Meta:
        app = 'Personel'
        verbose_name = _(u"Kadro")
        verbose_name_plural = _(u"Kadrolar")
        list_fields = ['durum', 'unvan', 'aciklama']
        search_fields = ['unvan', 'derece']
        list_filters = ['durum']

    def __unicode__(self):
        # tn: Kadro bilgileri gösterilirken kullanılan mesaj
        return gettext(u"%(no)s - %(unvan)s %(derece)s. derece") % {
            'no': self.kadro_no,
            'unvan': self.get_unvan_display(),
            'derece': self.derece
        }


class Izin(Model):
    """İzin Modeli

    Personelin ücretli izin bilgilerini içeren modeldir.

    """

    tip = field.Integer(_(u"Tip"), choices="izin")
    baslangic = field.Date(_(u"Başlangıç"), format="%d.%m.%Y")
    bitis = field.Date(_(u"Bitiş"), format="%d.%m.%Y")
    onay = field.Date(_(u"Onay"), format="%d.%m.%Y")
    adres = field.String(_(u"Geçireği Adres"))
    telefon = field.String(_(u"Telefon"))
    personel = Personel()
    vekil = Personel()

    class Meta:
        app = 'Personel'
        verbose_name = _(u"İzin")
        verbose_name_plural = _(u"İzinler")
        list_fields = ['tip', 'baslangic', 'bitis', 'onay', 'telefon']
        search_fields = ['tip', 'baslangic', 'onay']

    def __unicode__(self):
        return '%s %s' % (self.tip, self.onay)


class UcretsizIzin(Model):
    """Ücretsiz izin Modeli

    Personelin ücretsiz izin bilgilerini içeren modeldir.

    """

    tip = field.Integer(_(u"Tip"), choices="ucretsiz_izin")
    baslangic = field.Date(_(u"İzin Başlangıç Tarihi"), format="%d.%m.%Y")
    bitis = field.Date(_(u"İzin Bitiş Tarihi"), format="%d.%m.%Y")
    donus_tarihi = field.Date(_(u"Dönüş Tarihi"), format="%d.%m.%Y")
    donus_tip = field.Integer(_(u"Dönüş Tip"))
    onay_tarihi = field.Date(_(u"Onay Tarihi"), format="%d.%m.%Y")
    personel = Personel()

    class Meta:
        app = 'Personel'
        verbose_name = _(u"Ücretsiz İzin")
        verbose_name_plural = _(u"Ücretsiz İzinler")
        list_fields = ['tip', 'baslangic', 'bitis', 'donus_tarihi']
        search_fields = ['tip', 'onay_tarihi']

    def __unicode__(self):
        return '%s %s' % (self.tip, self.onay_tarihi)


class Atama(Model):
    """Atama Modeli

    Personelin atama bilgilerini içeren modeldir.

    """

    ibraz_tarihi = field.Date(_(u"İbraz Tarihi"), index=True, format="%d.%m.%Y")
    durum = HitapSebep()
    nereden = field.Integer(_(u"Nereden"), index=True)  # modele baglanacak.
    atama_aciklama = field.String(_(u"Atama Açıklama"), index=True)
    goreve_baslama_tarihi = field.Date(_(u"Göreve Başlama Tarihi"), index=True, format="%d.%m.%Y")
    goreve_baslama_aciklama = field.String(_(u"Göreve Başlama Açıklama"), index=True)
    sicil_no = field.String(_(u"Sicil No"))
    kadro = Kadro()
    personel = Personel()
    # Arama için eklendi, Orjinali personelde tutulacak
    hizmet_sinifi = field.Integer(_(u"Hizmet Sınıfı"), index=True, choices="hizmet_sinifi")

    class Meta:
        app = 'Personel'
        verbose_name = _(u"Atama")
        verbose_name_plural = _(u"Atamalar")
        list_fields = ['hizmet_sinifi', 'goreve_baslama_tarihi', 'ibraz_tarihi',
                       'durum']
        search_fields = ['hizmet_sinif', 'statu']

    def __unicode__(self):
        return '%s %s %s' % (self.personel.kurum_sicil_no,
                             self.goreve_baslama_tarihi, self.ibraz_tarihi)

    @classmethod
    def personel_guncel_atama(cls, personel):
        """
        Personelin goreve_baslama_tarihi ne göre son atama kaydını döndürür.

        Returns:
            Atama örneği (instance)

        """

        return cls.objects.set_params(
            sort='goreve_baslama_tarihi desc').filter(personel=personel)[0]

    @classmethod
    def personel_ilk_atama(cls, personel):
        """
        Personelin goreve_baslama_tarihi ne göre ilk atama kaydını döndürür.

        Returns:
            Atama örneği (instance)

        """

        return cls.objects.set_params(
            sort='goreve_baslama_tarihi asc').filter(personel=personel)[0]

    def post_save(self):
        # Personel modeline arama için eklenen kadro_derece set edilecek
        self.personel.kadro_derece = self.kadro.derece
        self.personel.save()

        # Atama sonrası kadro dolu durumuna çekilecek
        self.kadro.durum = 4
        self.kadro.save()

    def pre_save(self):
        self.hizmet_sinifi = self.personel.hizmet_sinifi
        # Atama kaydetmeden önce kadro boş durumuna çekilecek
        self.kadro.durum = 2
        self.kadro.save()

    def post_delete(self):
        # Atama silinirse kadro boş duşuma çekilecek
        self.kadro.durum = 2
        self.kadro.save()

        # personelin kadro derecesi 0 olacak
        self.personel.kadro_derece = 0
        self.personel.save()



# -*-  coding: utf-8 -*-
# Copyright (C) 2015 ZetaOps Inc.
#
# This file is licensed under the GNU General Public License v3
# (GPLv3).  See LICENSE.txt for details.
#

"""Kadro İşlemleri

Kadro İşlemleri İş Akışı 5 adimdan olusmaktadir:

    * Kadro Listele
    * Saklı Kadro Ekle
    * Kaydet
    * Kadro durumunu Saklı veya İzinli yap
    * Kadro Sil Onay
    * Kadro Sil

Bu iş akışı, CrudView nesnesi genişletilerek(extend) işletilmektedir.
Adımlar arası geçiş CrudView'ın aksine otomatik değil, manuel olarak
yapılmaktadır.

Her adım başına kullanılan metodlar şu şekildedir:

Kadro Listele:
   CrudView list metodu kullanılmıştır. Liste ekranında CrudView
   standart filtreleme ve arama özellikleri kullanılmaktadır.
   Listenin her bir öğesi için object_actions filtreleri @obj_filter
   dekoratörleri yardımıyla özelleştirilmiştir.

   Kadro işlemleri kurallarına göre sadece saklı kadrolar eklenebilmekte
   veya silinebilmektedir. Bu sebeple 'sil' eylemi sadece bu türdeki
   kadrolar icin aktifdir.

   Saklı / İzinli Yap butonu ise sadece saklı veya izinli kadrolar için
   görünürdür.

Saklı Kadro Ekle:
   Kadrolar sadece ve sadece saklı olarak sisteme eklenebilirler. Bu
   amaçla iş akışı adımlarına CrudView add_edit_form metodunun
   çağrıldığı adımın hemen ardından kaydet metodu konulmuştur. Bu metod
   durum alanını SAKLI olmaya zorlayarak nesneyi kaydeder.

Kaydet:
   WF'nin 2. adımından gelen data CrudView'in set_form_data_to_object
   metoduyla bir Kadro instance oluşturularak aktarılır.

   Durum alanı saklı (1) olarak sabitlenip kaydedilir.

Kadro Durumunu Sakli veya Izinli yap:
   Bunun icin özel bir metod eklenmistir: sakli_izinli_degistir. Bu
   üzerinde işlem yapılan kadronun durumu arasında geçis yapar.

Kadro Sil
   Sadece durumu sakli (1) olan kadrolar silinebilir. Bunun için kadro
   sil metodunda bu kontrol yapilir ve delete metodu çalıştırılır.

Kadro Sil Onay
   Silme işlemi için onay adımıdır.

"""
from pyoko.exceptions import ObjectDoesNotExist
from ulakbus.lib.personel import terfi_tarhine_gore_personel_listesi
from zengine.forms import JsonForm
from zengine.forms import fields
from zengine.views.crud import CrudView, obj_filter
from zengine.lib.translation import gettext as _, gettext_lazy, format_datetime, format_date
from ulakbus.models import Personel
from pyoko import ListNode
from dateutil.relativedelta import relativedelta
import datetime


class KadroObjectForm(JsonForm):
    """KadoIslemleri için object form olarak kullanılacaktır.

    Meta değiştirilerek, formlardan durum alanı çıkarılmış, ve form
    alanları iki gruba ayrılmıştır.

    Formun sadece bir kaydet butonu mevcuttur.

    """

    class Meta:
        exclude = ['durum', ]

        grouping = [
            {
                "layout": "4",
                "groups": [
                    {
                        "group_title": gettext_lazy(u"Ünvan ve Derece"),
                        "items": ['unvan', 'derece', 'unvan_aciklama'],
                        "collapse": True,
                    }
                ]
            },
            {
                "layout": "4",
                "groups": [
                    {
                        "group_title": gettext_lazy(u"Diğer"),
                        "items": ['kadro_no', 'aciklama', 'birim_id']
                    }
                ]
            },
            {
                "layout": "7",
                "groups": [
                    {
                        "items": ['kaydet']
                    }
                ]
            }
        ]

    save_edit = fields.Button(gettext_lazy(u"Kaydet"))


class KadroIslemleri(CrudView):
    """Kadro İşlemleri

    Kado işlemleri için kullanacağımız temel model Kadro modelidir.
    Meta.model bu amaçla kullanılmıştır. Aynı şekilde Meta içerisinde
    yer alan object_actions iş akışı boyunca özel olarak doldurulacağı
    için burada boşaltılmıştır.

    """

    # Kadro Durumları
    SAKLI = 1
    IZINLI = 2
    BOS = 3
    DOLU = 4

    class Meta:

        # CrudViev icin kullanilacak temel Model
        model = 'Kadro'

        # ozel bir eylem listesi hazirlayacagiz. bu sebeple listeyi bosaltiyoruz.
        # kayit tipine bagli olarak ekleyecegimiz eylemleri .append() ile ekleyecegiz
        object_actions = [
            # boş, action dictler sonraki adımlarda bu listeye aşağıdaki biçimde eklenmeli:
            # {'fields': [0, ], 'cmd': 'show', 'mode': 'normal', 'show_as': 'link'},
        ]

    def __init__(self, current):
        """Standart ObjectForm nesnesini değil, hemen yukarıda tanımladığımız
        KadroObjectForm nesnesini kullan. Bu atamayı yapıp, üst sınıfın init
        metodu çağrılır.

        """

        self.ObjectForm = KadroObjectForm
        super(KadroIslemleri, self).__init__(current)

    def kadro_kaydet(self):
        """Formdan gelen dataları Kadro örneğine doldurur ve kaydeder.

        Kadrolar yanlızca SAKLI olarak kayıt edilebilecekleri için kaydetmeden
        önce durum alanı SAKLI yapılır.

        İş akışı bu adımdan sonra sona eriyor. İş akışını yeni bir token ile
        yenilemek için reset metodunu çağırıyoruz.

        """

        # Formdan gelen datayi, instance a gecir.
        self.set_form_data_to_object()

        # Durumu ne olursa olsun (sakli) yap!..
        self.object.durum = self.SAKLI

        # Kadroyu kaydet
        self.object.save()

        # İş akışını yenile
        self.reset()

    class SilOnayForm(JsonForm):
        evet = fields.Button(gettext_lazy(u"Evet"), cmd='kadro_sil')
        hayir = fields.Button(gettext_lazy(u"Hayır"))

    def kadro_sil_onay_form(self):
        """
        Silme işlemi için onay adımı. Kadronun detaylı açıklaması listelenir ve
        onay vermesi beklenir.

        """

        unvan = self.object.get_unvan_display()
        aciklama = self.object.aciklama
        kadro_no = self.object.kadro_no

        self.current.task_data['object_id'] = self.object.key

        _form = self.SilOnayForm(title=" ")
        _form.help_text = _(u"""Akademik unvanı: **%(unvan)s**
        Kadro numarası: **%(kadro)s**
        Açıklaması: **%(aciklama)s**

        bilgilerine sahip kadroyu silmek istiyor musunuz ?""") % {
            'unvan': unvan, 'kadro': kadro_no, 'aciklama': aciklama,
        }
        self.form_out(_form)

    def kadro_sil(self):
        """
        Saklı kontolü yaparak, silme işlemini gerçekleştirir.

        """
        # TODO: Sakli kadronun silinme denemesi loglanacak.
        assert self.object.durum == self.SAKLI, "attack detected, should be logged/banned"
        self.object.blocking_delete()
        del self.current.task_data['object_id']

    def sakli_izinli_degistir(self):
        """Saklı İzinli Değiştir

        Durum degerini SAKLI ve IZINLI arasinda degistir.

        sakliysa izinli yap 3 - SAKLI = IZINLI
        izinliyse sakli yap 3 - IZINLI = SAKLI

        """

        self.object.durum = 3 - self.object.durum
        self.object.save()

    @obj_filter
    def sakli_kadro(self, obj, result):
        """Saklı Kadro Filtresi

        Saklı kadro listesinde yer alan her bir öğeye İzinli Yap butonu ekler.

        Args:
            obj: Kadro instance
            result: dict

        """

        if obj.durum == self.SAKLI:
            result['actions'].extend([
                {'name': _(u'Sil'), 'cmd': 'kadro_sil_onay_form', 'show_as': 'button'},
                {'name': _(u'İzinli Yap'), 'cmd': 'sakli_izinli_degistir', 'show_as': 'button'}])

    @obj_filter
    def izinli_kadro(self, obj, result):
        """İzinli Kadro Filtresi

        İzinli Kadro listesinde yer alan her bir öğeye Saklı Yap butonu
        ekler.

        Args:
            obj: Kadro instance
            result: dict

        """

        if obj.durum == self.IZINLI:
            result['actions'].append(
                {'name': _(u'Sakli Yap'), 'cmd': 'sakli_izinli_degistir', 'show_as': 'button'})

    @obj_filter
    def duzenlenebilir_veya_silinebilir_kadro(self, obj, result):
        """Düzenlenebilir ve Silinebilir Kadro Filtresi

        Düzenlenebilir Kadro listesinde yer alan saklı veya izinli
        her bir öğeye Sil ve Düzenle butonu ekler.

        Args:
            obj: Kadro instance
            result: dict

        """

        if obj.durum in [self.SAKLI, self.IZINLI]:
            result['actions'].extend([
                {'name': _(u'Düzenle'), 'cmd': 'add_edit_form', 'show_as': 'button'},
            ])


class TerfiForm(JsonForm):
    class Meta:
        inline_edit = ['sec']

    class Personel(ListNode):
        key = fields.String("Key", hidden=True)
        sec = fields.Boolean(gettext_lazy(u"Seç"), type="checkbox")
        tckn = fields.String(gettext_lazy(u"TCK No"))
        ad_soyad = fields.String(gettext_lazy(u"Ad"))
        kadro_derece = fields.String(gettext_lazy(u"Kadro Derece"))

        #tn: Görev Aylığı
        gorev_ayligi = fields.String(gettext_lazy(u"GA"))
        #tn: Kazanılmış Hak
        kazanilmis_hak = fields.String(gettext_lazy(u"KH"))
        #tn: Emekli Muktesebat
        emekli_muktesebat = fields.String(gettext_lazy(u"EM"))

        # tn: Yeni Görev Aylığı
        yeni_gorev_ayligi = fields.String(gettext_lazy(u"Yeni GA"))
        #tn: Yeni Kazanılmış Hak
        yeni_kazanilmis_hak = fields.String(gettext_lazy(u"Yeni KH"))
        #tn: Yeni Emekli Muktesebat
        yeni_emekli_muktesebat = fields.String(gettext_lazy(u"Yeni EM"))

    def generate_form(self):
        """
        Generates form with given data ``personel_data``

        """

        for p_key, p_data in self.context.task_data["personeller"].items():
            self.Personel(
                key=p_key,
                sec=False,
                tckn=p_data["tckn"],
                ad_soyad="%s %s" % (p_data["ad"], p_data["soyad"]),
                kadro_derece=p_data["kadro_derece"],

                gorev_ayligi="%s/%s(%s)" % (
                    p_data["guncel_gorev_ayligi_derece"], p_data["guncel_gorev_ayligi_kademe"],
                    p_data["gorunen_gorev_ayligi_kademe"]),

                kazanilmis_hak="%s/%s(%s)" % (
                    p_data["guncel_kazanilmis_hak_derece"], p_data["guncel_kazanilmis_hak_kademe"],
                    p_data["gorunen_kazanilmis_hak_kademe"]),

                emekli_muktesebat="%s/%s(%s)" % (
                    p_data["guncel_emekli_muktesebat_derece"],
                    p_data["guncel_emekli_muktesebat_kademe"],
                    p_data["gorunen_emekli_muktesebat_kademe"]),

                yeni_gorev_ayligi="%s/%s(%s)" % (
                    p_data["terfi_sonrasi_gorev_ayligi_derece"],
                    p_data["terfi_sonrasi_gorev_ayligi_kademe"],
                    p_data["terfi_sonrasi_gorunen_gorev_ayligi_kademe"]),

                yeni_kazanilmis_hak="%s/%s(%s)" % (
                    p_data["terfi_sonrasi_kazanilmis_hak_derece"],
                    p_data["terfi_sonrasi_kazanilmis_hak_kademe"],
                    p_data["terfi_sonrasi_gorunen_kazanilmis_hak_kademe"]),

                yeni_emekli_muktesebat="%s/%s(%s)" % (
                    p_data["terfi_sonrasi_emekli_muktesebat_derece"],
                    p_data["terfi_sonrasi_emekli_muktesebat_kademe"],
                    p_data["terfi_sonrasi_gorunen_emekli_muktesebat_kademe"])
            )


class TerfiDuzenleForm(JsonForm):
    class Meta:
        inline_edit = [
            'yeni_gorev_ayligi_derece', 'yeni_gorev_ayligi_kademe', 'yeni_gorev_ayligi_gorunen',
            'yeni_kazanilmis_hak_derece', 'yeni_kazanilmis_hak_kademe',
            'yeni_kazanilmis_hak_gorunen', 'yeni_emekli_muktesebat_derece',
            'yeni_emekli_muktesebat_kademe', 'yeni_emekli_muktesebat_gorunen']

        help_text = gettext_lazy(u"""
        GA: Görev Aylığı
        KH: Kazanılmış Hak
        EM: Emekli Muktesebat

        D: Derece
        K: Kademe
        G: Gorunen
        """)

    class Personel(ListNode):
        key = fields.String("Key", hidden=True)
        tckn = fields.String(gettext_lazy(u"T.C. No"))
        ad_soyad = fields.String(gettext_lazy(u"İsim"))
        kadro_derece = fields.String(gettext_lazy(u"Kadro Derece"))

        #tn: Görev Aylığı
        gorev_ayligi = fields.String(gettext_lazy(u"GA"))
        #tn: Kazanılmış Hak
        kazanilmis_hak = fields.String(gettext_lazy(u"KH"))
        #tn: Emekli Muktesebat
        emekli_muktesebat = fields.String(gettext_lazy(u"EM"))

        #tn: Yeni Görev Aylığı Derece
        yeni_gorev_ayligi_derece = fields.Integer(gettext_lazy(u"GAD"))
        #tn: Yeni Görev Aylığı Kademe
        yeni_gorev_ayligi_kademe = fields.Integer(gettext_lazy(u"GAK"))
        #tn: Yeni Görev Aylığı Görünen
        yeni_gorev_ayligi_gorunen = fields.Integer(gettext_lazy(u"GAG"))

        #tn: Kazanılmış Görev Aylığı Derece
        yeni_kazanilmis_hak_derece = fields.Integer(gettext_lazy(u"KHD"))
        #tn: Kazanılmış Görev Aylığı Derece
        yeni_kazanilmis_hak_kademe = fields.Integer(gettext_lazy(u"KHK"))
        #tn: Kazanılmış Görev Aylığı Derece
        yeni_kazanilmis_hak_gorunen = fields.Integer(gettext_lazy(u"KHG"))

        #tn: Emekli Muktesebat Derece
        yeni_emekli_muktesebat_derece = fields.Integer(gettext_lazy(u"EMD"))
        #tn: Emekli Muktesebat Derece
        yeni_emekli_muktesebat_kademe = fields.Integer(gettext_lazy(u"EMK"))
        #tn: Emekli Muktesebat Derece
        yeni_emekli_muktesebat_gorunen = fields.Integer(gettext_lazy(u"EMG"))

    devam = fields.Button(gettext_lazy(u"Devam Et"), cmd="kaydet")


class PersonelTerfiKriterleri(JsonForm):
    baslangic_tarihi = fields.Date(gettext_lazy(u"Başlangıç Tarihi"),
                                   default=datetime.date.today().strftime('d.%m.%Y'))

    bitis_tarihi = fields.Date(gettext_lazy(u"Bitiş Tarihi"), default=(
        datetime.date.today() + datetime.timedelta(days=15)).strftime('d.%m.%Y'))

    personel_turu = fields.Integer(gettext_lazy(u"Personel Türü"), choices=[(1, gettext_lazy(u"Akademik")), (2, gettext_lazy(u"İdari"))],
                                   default=2)

    devam = fields.Button(gettext_lazy(u"Sorgula"))


class TerfiListe(CrudView):
    class Meta:
        model = "Personel"

    def personel_kriterleri(self):
        _form = PersonelTerfiKriterleri(current=self.current,
                                        title=_(u"Terfisi Yapılacak Personel Kriterleri"))
        self.form_out(_form)

    def terfisi_gelen_personel_liste(self):

        try:
            self.current.task_data["personeller"]
        except KeyError:
            personel_turu = self.current.input['form']['personel_turu']

            baslangic_tarihi = datetime.datetime.strptime(
                self.current.input['form']['baslangic_tarihi'], '%d.%m.%Y')
            bitis_tarihi = datetime.datetime.strptime(
                self.current.input['form']['bitis_tarihi'], '%d.%m.%Y')
            self.current.task_data["personeller"] = terfi_tarhine_gore_personel_listesi(
                baslangic_tarihi=baslangic_tarihi, bitis_tarihi=bitis_tarihi,
                personel_turu=personel_turu)

        if self.current.task_data["personeller"]:
            _form = TerfiForm(current=self.current, title=_(u"Terfi İşlemi"))
            _form.generate_form()

            _form.kaydet = fields.Button(_(u"Onaya Gönder"), cmd="onaya_gonder")
            _form.duzenle = fields.Button(_(u"Terfi Düzenle"), cmd="terfi_liste_duzenle")

            self.form_out(_form)
            self.current.output["meta"]["allow_actions"] = False
            self.current.output["meta"]["allow_add_listnode"] = False
        else:
            datetime.datetime.today()
            self.current.output['msgbox'] = {
                'type': 'info', "title": _(u'Terfi Bekleyen Personel Bulunamadı'),
                "msg": _(u'%(baslangic)s - %(bitis)s tarih aralığında terfi bekleyen personel bulunamadı.') % {
                    'baslangic': format_datetime(baslangic_tarihi),
                    'bitis': format_datetime(bitis_tarihi),
                }
            }

    def terfi_liste_duzenle(self):
        _form = TerfiDuzenleForm()
        for p in self.current.input['form']['Personel']:
            if p['sec']:
                p_data = self.current.task_data['personeller'][p['key']]
                _form.Personel(
                    key=p_data['key'],
                    tckn=p_data["tckn"],
                    ad_soyad="%s %s" % (p_data["ad"], p_data["soyad"]),
                    kadro_derece=p_data["kadro_derece"],

                    gorev_ayligi="%s/%s(%s)" % (
                        p_data["guncel_gorev_ayligi_derece"], p_data["guncel_gorev_ayligi_kademe"],
                        p_data["gorunen_gorev_ayligi_kademe"]),

                    kazanilmis_hak="%s/%s(%s)" % (
                        p_data["guncel_kazanilmis_hak_derece"],
                        p_data["guncel_kazanilmis_hak_kademe"],
                        p_data["gorunen_kazanilmis_hak_kademe"]),

                    emekli_muktesebat="%s/%s(%s)" % (
                        p_data["guncel_emekli_muktesebat_derece"],
                        p_data["guncel_emekli_muktesebat_kademe"],
                        p_data["gorunen_emekli_muktesebat_kademe"]),

                    yeni_gorev_ayligi_derece=p_data["terfi_sonrasi_gorev_ayligi_derece"],
                    yeni_gorev_ayligi_kademe=p_data["terfi_sonrasi_gorev_ayligi_kademe"],
                    yeni_gorev_ayligi_gorunen=p_data["terfi_sonrasi_gorunen_gorev_ayligi_kademe"],

                    yeni_kazanilmis_hak_derece=p_data["terfi_sonrasi_kazanilmis_hak_derece"],
                    yeni_kazanilmis_hak_kademe=p_data["terfi_sonrasi_kazanilmis_hak_kademe"],
                    yeni_kazanilmis_hak_gorunen=p_data[
                        "terfi_sonrasi_gorunen_kazanilmis_hak_kademe"],

                    yeni_emekli_muktesebat_derece=p_data["terfi_sonrasi_emekli_muktesebat_derece"],
                    yeni_emekli_muktesebat_kademe=p_data["terfi_sonrasi_emekli_muktesebat_kademe"],
                    yeni_emekli_muktesebat_gorunen=p_data[
                        "terfi_sonrasi_gorunen_emekli_muktesebat_kademe"]

                )

        self.form_out(_form)

    def terfi_duzenle_kaydet(self):
        for p in self.current.input['form']['Personel']:
            p_data = self.current.task_data['personeller'][p['key']]

            p_data["terfi_sonrasi_gorev_ayligi_derece"] = p["yeni_gorev_ayligi_derece"]
            p_data["terfi_sonrasi_gorev_ayligi_kademe"] = p["yeni_gorev_ayligi_kademe"]
            p_data["terfi_sonrasi_gorunen_gorev_ayligi_kademe"] = p["yeni_gorev_ayligi_gorunen"]

            p_data["terfi_sonrasi_kazanilmis_hak_derece"] = p["yeni_kazanilmis_hak_derece"]
            p_data["terfi_sonrasi_kazanilmis_hak_kademe"] = p["yeni_kazanilmis_hak_kademe"]
            p_data["terfi_sonrasi_gorunen_kazanilmis_hak_kademe"] = p["yeni_kazanilmis_hak_gorunen"]

            p_data["terfi_sonrasi_emekli_muktesebat_derece"] = p["yeni_emekli_muktesebat_derece"]
            p_data["terfi_sonrasi_emekli_muktesebat_kademe"] = p["yeni_emekli_muktesebat_kademe"]
            p_data["terfi_sonrasi_gorunen_emekli_muktesebat_kademe"] = p[
                "yeni_emekli_muktesebat_gorunen"]

    # todo: lane geicisi
    def mesaj_goster(self):

        msg = {"title": _(u'Personeller Onay Icin Gonderildi!'),
               "body": _(u'Talebiniz Basariyla iletildi.')}
        # workflowun bu kullanıcı için bitişinde verilen mesajı ekrana bastırır

        self.current.task_data['LANE_CHANGE_MSG'] = msg

    def onay_kontrol(self):
        _form = TerfiForm(current=self.current, title=_(u"Terfi İşlemi"))
        _form.generate_form()
        _form.Meta.inline_edit = []

        _form.kaydet = fields.Button(title=_(u"Onayla"), cmd="terfi_yap")
        _form.duzenle = fields.Button(title=_(u"Reddet"), cmd="red_aciklamasi_yaz")

        self.form_out(_form)
        self.current.output["meta"]["allow_actions"] = False
        self.current.output["meta"]["allow_add_listnode"] = False

    def red_aciklamasi_yaz(self):
        _form = JsonForm(title=_(u"Terfi İşlemi Reddedildi."))
        _form.Meta.help_text = _(u"""Terfi işlemini onaylamadınız. İlgili personele bir açıklama
                                  yazmak ister misiniz?""")
        _form.red_aciklama = fields.String(_(u"Açıklama"))
        _form.devam = fields.Button(_(u"Devam Et"))
        self.form_out(_form)

    def red_aciklamasi_kaydet(self):
        self.current.task_data["red_aciklama"] = self.current.input['form']['red_aciklama']

    def terfi_yap(self):
        for key, p_data in self.current.task_data['personeller'].items():
            try:
                personel = Personel.objects.get(key)

                personel.gorev_ayligi_derece = p_data["terfi_sonrasi_gorev_ayligi_derece"]
                personel.gorev_ayligi_kademe = p_data["terfi_sonrasi_gorev_ayligi_kademe"]

                personel.kazanilmis_hak_derece = p_data["terfi_sonrasi_kazanilmis_hak_derece"]
                personel.kazanilmis_hak_kademe = p_data["terfi_sonrasi_kazanilmis_hak_kademe"]

                personel.emekli_muktesebat_derece = p_data["terfi_sonrasi_emekli_muktesebat_derece"]
                personel.emekli_muktesebat_kademe = p_data["terfi_sonrasi_emekli_muktesebat_kademe"]

                personel.kh_sonraki_terfi_tarihi = personel.kh_sonraki_terfi_tarihi + relativedelta(
                    years=1)

                personel.save()

            except ObjectDoesNotExist:
                # TODO: LOG for sysadmin. Artik olmayan bir personel uzerinde terfi islemi..
                pass

    # todo: lane geicisi
    def taraflari_bilgilendir(self):
        msg = {"title": _(u'Personel Terfi Islemi Onaylandi!'),
               "body": _(u'Onay Belgesi icin Personel Islerine Gonderildi.')}
        self.current.output['msgbox'] = msg
        self.current.task_data['LANE_CHANGE_MSG'] = msg

    def onay_belgesi_uret(self):
        self.current.output['msgbox'] = {
            'type': 'info',
            'title': _(u'Terfi İşlemleri Onay Belgesi!'),
            'msg': _(u'Toplu terfi İşleminiz Onaylandı')
        }


class GorevSuresiForm(JsonForm):
    """ 
        Akademik personel görev süresini uzatma işlemi için kullanılan
        JsonForm dan türetilmiş bir form sınıfıdır.
    """

    def __init__(self, **kwargs):
        """ 
            form nesnesi üretilirken girilen parametrelerin form elemanlarının
            default değeri olarak kullanıla bilmesi amacıyla yazılan constructor
            metoddur. Akademik personelin görev süresi Atama modelinde tutulduğu
            için form nesnesi instance üretilirken ilgili atama nın id ne
            ihtiyacımız bulunmaktadır.
        """

        self.gorev_suresi_bitis = fields.Date(_(u"Görev Süresi Bitiş Tarihi"),
                                              default=kwargs.pop('gorev_suresi_bitis_tarihi'))
        self.personel_id = fields.String("personel_id", hidden=True,
                                         default=kwargs.pop('personel_id'))

        # Üst sınıfın constructor metodu çağrılmaktadır.        
        super(GorevSuresiForm, self).__init__()

    kaydet = fields.Button(gettext_lazy(u"Kaydet"), cmd="kaydet")


class GorevSuresiUzat(CrudView):
    class Meta:
        model = "Personel"

    """ 
        Görev süresi uzatma işlemini gerçekleştiren CrudView den türetilmiş
        bir sınıftır.
    """

    def gorev_suresi_form(self):
        """ 
        Öncelikle anasayfadaki personel seçim formundan seçilen personelin
        id si elde edilir. Personel id ile atama kaydı elde edilir.
        Eğer personel akademik personel değilse hata mesajı görüntülenir.
        Her akademik personele ait sadece bir adet atama kaydı bulunabilir.
        Elde edilen atama nesnesinden çekilen görev süresi bitiş tarihi
        form nesnesi instance üretilirken parametre olarak verilir.
        Son olarak da form görüntülenir.
        """

        try:
            personel = Personel.objects.get(self.current.input["id"])

            if personel.personel_turu == 1:
                if type(personel.gorev_suresi_bitis) is datetime.date:
                    gorev_suresi_bitis = format_date(personel.gorev_suresi_bitis)
                else:
                    gorev_suresi_bitis = None

                _form = GorevSuresiForm(current=self.current, title=_(u"Görev Süresi Uzat"),
                                        gorev_suresi_bitis_tarihi=gorev_suresi_bitis,
                                        personel_id=personel.key)
                self.form_out(_form)
            else:
                self.current.output['msgbox'] = {
                    'type': 'info', "title": _(u'HATA !'),
                    "msg": _(u'%(ad)s %(soyad)s akademik bir personel değildir.') % {
                        'ad': personel.ad,
                        'soyad': personel.soyad,
                    }
                }

        except ObjectDoesNotExist:
            self.current.output["msgbox"] = {
                'type': "info", "title": _(u"HATA !"),
                "msg": _(u"%(ad)s %(soyad)s e ait bir atama kaydı bulunamadı") % {
                    'ad': personel.ad,
                    'soyad': personel.soyad,
                }
            }

    def kaydet(self):
        """ 
        Formdan gelen personel id ile personel kaydı elde edilir. Sonrasındada
        görev süresi başlama ve bitiş tarihleri değiştirilerek kaydedilir.
        Yeni görev süresi başlama tarihi işlemin yapıldığı tarih,
        yeni görev süresi bitiş tarihi formdan gelen tarih olur.
        """
        personel = Personel.objects.get(self.current.input["form"]["personel_id"])
        personel.gorev_suresi_baslama = datetime.date.today()
        personel.gorev_suresi_bitis = self.current.input["form"]["gorev_suresi_bitis"]
        personel.save()



# -*-  coding: utf-8 -*-

# Copyright (C) 2015 ZetaOps Inc.
#
# This file is licensed under the GNU General Public License v3
# (GPLv3).  See LICENSE.txt for details.

"""HITAP Hizmet Cetveli Sorgula

Hitap üzerinden personelin hizmet kaydı bilgilerinin sorgulamasını yapar.

"""

from ulakbus.services.personel.hitap.hitap_sorgula import HITAPSorgula


class HizmetCetveliGetir(HITAPSorgula):
    """
    HITAP Sorgulama servisinden kalıtılmış
    Hizmet Kaydı Bilgisi Sorgulama servisi

    """
    HAS_CHANNEL = True

    def handle(self):
        """
        Servis çağrıldığında tetiklenen metod.

        Attributes:
            service_name (str): İlgili Hitap sorgu servisinin adı
            bean_name (str): Hitap'tan gelen bean nesnesinin adı
            service_dict (dict): Hitap servisinden gelen kayıtların alanları,
                    ``HizmetKayitlari`` modelinin alanlarıyla eşlenmektedir.
                    Filtreden geçecek tarih alanları listede tutulmaktadır.

        """

        self.service_name = 'HizmetCetvelSorgula'
        self.bean_name = 'HizmetCetveliServisBean'
        self.service_dict = {
            'fields': {
                'tckn': 'tckn',
                'baslama_tarihi': 'baslamaTarihi',
                'bitis_tarihi': 'bitisTarihi',
                'emekli_derece': 'emekliDerece',
                'emekli_kademe': 'emekliKademe',
                'gorev': 'gorev',
                'unvan_kod': 'unvanKod',
                'hizmet_sinifi': 'hizmetSinifi',
                'kayit_no': 'kayitNo',
                'kazanilmis_hak_ayligi_derece': 'kazanilmisHakAyligiDerece',
                'kazanilmis_hak_ayligi_kademe': 'kazanilmisHakAyligiKademe',
                'odeme_derece': 'odemeDerece',
                'odeme_kademe': 'odemeKademe',
                'emekli_ek_gosterge': 'emekliEkGosterge',
                'kadro_derece': 'kadroDerece',
                'kazanilmis_hak_ayligi_ekgosterge': 'kazanilmisHakAyligiEkGosterge',
                'odeme_ekgosterge': 'odemeEkGosterge',
                'sebep_kod': 'sebepKod',
                'ucret':'ucret',
                'yevmiye': 'yevmiye',
                'kurum_onay_tarihi': 'kurumOnayTarihi'
            },
            'date_filter': ['baslama_tarihi', 'bitis_tarihi', 'kurum_onay_tarihi'],
            'required_fields': ['tckn']
        }
        super(HizmetCetveliGetir, self).handle()

    def custom_filter(self, hitap_dict):
        """
        Hitap sözlüğüne uygulanacak ek filtreleri gerçekleştirir.

        Args:
            hitap_dict (List[dict]): Hitap verisini yerele uygun biçimde tutan sözlük listesi

        """

        for record in hitap_dict:
            record['hizmet_sinifi'] = self.hizmet_sinifi_int_kontrol(record['hizmet_sinifi'])

    def hizmet_sinifi_int_kontrol(self, hizmet_sinifi):
        """
        Hitap Hizmet Cetveli sorgulama servisinin,
        hem 1, 2, 3 ... 29 gibi tam sayı (servisten string olarak geliyor) değerleri
        hem de GİH, MİAH, ... SÖZ gibi string değerleri alabilen,
        hizmetSinifi alanının, her koşulda tam sayı olarak elde edilmesini sağlar.

        Args:
            hizmet_sinifi (str): Hizmet Cetveli hizmet sınıfı bilgisi

        Returns:
            int: Hizmet sınıfı tam sayı değeri.

        Raises:
            ValueError: Geçersiz hizmet sınıfı değeri.

        """

        hizmet_siniflari = {
            u"GİH": 1,
            u"MİAH": 2,
            u"SH": 3,
            u"TH": 4,
            u"EÖH": 5,
            u"AH": 6,
            u"EH": 7,
            u"DH": 8,
            u"YH": 9,
            u"MİT": 10,
            u"AHS": 11,
            u"BB": 12,
            u"CU": 13,
            u"CUM": 14,
            u"DE": 15,
            u"DVS": 16,
            u"HS": 17,
            u"MB": 18,
            u"MV": 19,
            u"ÖÜ": 20,
            u"SAY": 21,
            u"TBM": 22,
            u"TRT": 23,
            u"TSK": 24,
            u"YÖK": 25,
            u"YSH": 26,
            u"ÖGO": 27,
            u"ÖY": 28,
            u"SÖZ": 29
        }

        try:
            # return if it is int
            return int(hizmet_sinifi)

        except ValueError:
            # if not, find its value (int) in dict (key type is 'suds.sax.text.Text')
            try:
                return hizmet_siniflari[hizmet_sinifi.strip()]
            except KeyError:
                # TODO: admin'e bildirim gitmesi lazim
                self.logger.exception("Hizmet Sinifini (%s) Kontrol Edin!", hizmet_sinifi)
                return 0



# -*-  coding: utf-8 -*-

# Copyright (C) 2015 ZetaOps Inc.
#
# This file is licensed under the GNU General Public License v3
# (GPLv3).  See LICENSE.txt for details.

from pyoko import ListNode
from zengine.views.crud import CrudView
from zengine.forms import fields
from zengine.forms import JsonForm
from zengine.lib.translation import gettext as _, gettext_lazy
from ulakbus.models.ogrenci import Okutman, Donem, Takvim, Unit, Personel
from ulakbus.models.personel import Izin
from datetime import datetime
import calendar
from collections import OrderedDict
from ulakbus.lib.date_time_helper import AYLAR, ay_listele
from ulakbus.views.reports import ders_ucreti_hesaplama as DU

guncel_yil = datetime.now().year
guncel_ay = datetime.now().month

# Guncel donem ve 5 onceki yili tuple halinde YIL listesinde tutar.
YIL = []
for i in range(5):
    YIL.append((i, guncel_yil - i))


class TarihForm(JsonForm):
    """
    Puantaj tablosu hazırlanırken ay ve yıl seçiminde
    kullanılan form.
    """

    class Meta:
        title = gettext_lazy(u'Puantaj Tablosu Hazırlamak İstediğiniz Yıl ve Ayı Seçiniz')

    yil_sec = fields.String(_(u'Yıl Seçiniz'), choices=YIL, default=0)
    ay_sec = fields.String(_(u'Ay Seçiniz'), choices=ay_listele, default=guncel_ay)


class OkutmanListelemeForm(JsonForm):
    """
    Puantaj tablosu hazırlanacak olan okutmanların listesini
    gösteren form.
    """

    class Meta:
        inline_edit = ['secim']

    class OkutmanListesi(ListNode):
        secim = fields.Boolean(_(u"Seçim"), type="checkbox")
        okutman = fields.String(_(u'Okutman'), index=True)
        key = fields.String(hidden=True)


class DersUcretiHesaplama(CrudView):
    """
    Tüm okutmanların aylık olarak girdikleri derslerin
    ve ek derslerin saatlerini hesaplamaya yarayan iş akışı.
    Izin gunleri ve resmi tatil gunlerini de dikkate alir.
    """

    def tarih_sec(self):
        """
        Puantaj tablosunun hesaplanacağı ay ve yılı
        seçmeye yarar.
        """

        _form = TarihForm(current=self.current)
        _form.sec = fields.Button(_(u"İlerle"))
        self.form_out(_form)

    def donem_kontrol(self):
        """
        Seçilen ay ve yıla ait dönem olup olmadığını kontrol eder.
        """

        self.current.task_data["yil"] = YIL[self.input['form']['yil_sec']][1]
        self.current.task_data["ay"] = self.input['form']['ay_sec']
        # guncel olan ayın ismi getirilir.
        self.current.task_data["ay_isim"] = AYLAR[self.input['form']['ay_sec'] - 1][1]

        takvim = calendar.monthrange(self.current.task_data["yil"], self.current.task_data["ay"])
        donem_list = Donem.takvim_ayina_rastlayan_donemler(self.current.task_data["yil"], self.current.task_data["ay"], takvim)

        if len(donem_list) > 0:
            self.current.task_data['donem_sayi'] = True
        else:
            self.current.task_data['donem_sayi'] = False

    def donem_secim_uyari(self):
        """
        Eğer seçilen ay ve yıla ait dönem bulunamamışsa uyarı verir.
        2016 Güz dönemindeyken 2016 Temmuz ayı (Yaz Dönemi) için bir sorgu
        istenirse daha o dönem açılmadığından hata verecektir.
        """

        _form = JsonForm(current=self.current, title=_(u"Dönem Bulunamadı"))
        _form.help_text = _(u"""Seçtiğiniz yıl ve aya ait dönem bulunamadı. Tarih
                          seçimine geri dönmek için Geri Dön butonuna, işlemi
                          iptal etmek için İptal butonuna basabilirsiniz.""")
        _form.geri_don = fields.Button(_(u"Geri Dön"), flow='tarih_sec')
        _form.iptal = fields.Button(_(u"İptal"))
        self.form_out(_form)

    def islem_iptali_bilgilendir(self):

        self.current.output['msgbox'] = {
            'type': 'info', "title": _(u'Durum Mesajı'),
            "msg": _(u'Puantaj cetveli görüntüleme işleminiz iptal edilmiştir.')
        }

    def ders_saati_turu_secme(self):
        """
        Ders Ücreti ya da Ek Ders Ücreti hesaplarından birini seçmeye yarar.
        """

        _form = JsonForm(current=self.current, title=_(u"Öğretim Görevlileri "
                                                       u"Puantaj Tablosu Hesaplama "
                                                       u"Türü Seçiniz"))

        _form.ders = fields.Button(_(u"Ders Ücreti Hesapla"), cmd='ders_ucreti_hesapla')
        _form.ek_ders = fields.Button(_(u"Ek Ders Ücreti Hesapla"), cmd='ek_ders_ucreti_hesapla')
        self.form_out(_form)

    def ders_ucreti_hesapla(self):

        self.current.task_data["control"] = True

    def ek_ders_ucreti_hesapla(self):
        self.current.task_data["control"] = False

    def ucret_hesapla(self):
        """
        Seçilen ay ve yıla göre, seçilen her bir öğretim elemanının
        resmi tatil ve izinleri dikkate alarak aylık ders saati planlamasını
        yapar
        """
        okutman_personel = Personel.objects.get(user=self.current.user)
        okutman = Okutman.objects.get(personel=okutman_personel)
        okutman_adi = okutman.ad + ' ' + okutman.soyad

        yil = self.current.task_data["yil"]  # girilen yil
        ay = self.current.task_data["ay"]  # girilen ayin orderi
        ay_isim = self.current.task_data["ay_isim"]  # ayin adi

        takvim = calendar.monthrange(yil, ay)
        # verilen yıl ve aya göre tuple şeklinde ayın ilk gününü
        # ve ayın kaç gün sürdüğü bilgisini döndürür.
        # Ayın ilk günü = 0-6 Pazt-Pazar
        # 2016 yılı Temmuz ayı için = (4,31)

        birim_no = self.current.role.unit.yoksis_no  # rolden gelecek
        birim_unit = Unit.objects.get(yoksis_no=birim_no)

        # Secilen ay hangi donemleri kapsiyor, kac donemi kapsıyorsa
        # o donemleri dondürür.
        donem_list = Donem.takvim_ayina_rastlayan_donemler(yil, ay, takvim)

        # Resmi tatilerin gununu (23, 12, 8) gibi ve doneme gore akademik takvim donduruyor
        resmi_tatil_list, akademik_takvim_list = Takvim.resmi_tatil_gunleri_getir(donem_list, birim_unit, yil, ay)

        # Kapsadığı dönemlere göre ders baslangıc ve bitis tarihlerini
        # baz alarak kapsadığı her bir dönemin seçilen ayda hangi gün
        # aralıklarını kapsadığı bilgisini tuple olarak dondurur. Bir
        # dönemi kapsıyorsa bir tuple, iki dönemi kapsıyorsa iki tuple
        # döner. (4,12) (19,31) gibi
        tarih_araligi = DU.donem_aralik_dondur(donem_list, yil, ay, takvim, akademik_takvim_list)

        object_list = ['Öğretim Elemanı']
        tarih_list = list(range(1, takvim[1] + 1))
        for tarih in tarih_list:
            object_list.append(' ' + str(tarih))

        # object_list = ['Ogretim Elemani', 'bir', 'iki', 'uc']
        _form = JsonForm(current=self.current)

        kontrol = self.current.task_data["control"]
        if kontrol:
            _form.title = _(u"%(ad)s - %(birim)s %(yil)s-%(ay)s AYI DERS PUANTAJ TABLOSU") % {
                'ad': okutman_adi.upper(),
                'birim': birim_unit.name,
                'yil': yil,
                'ay': ay_isim.upper()}
            ders_saati_turu = _(u'Ders Saati')

        else:
            _form.title = _(u"%s - %s %s-%s AYI EK DERS PUANTAJ TABLOSU") % (
            okutman_adi.upper(), birim_unit.name, yil, ay_isim.upper())
            ders_saati_turu = _(u'Ek Ders Saati')

        object_list.append(ders_saati_turu)
        self.output['objects'] = [object_list]

        data_list = OrderedDict({})

        # Seçilen okutmanın İzin ve Ücretsiz İzinlerini dikkate alarak, verilen ayda
        # hangi günler izinli olduğunu liste halinde döndürmeye yarayan method
        # [17,18,19] gibi
        personel_izin_list = Izin.personel_izin_gunlerini_getir(okutman, yil, ay)

        # Verilen döneme ve okutmana göre, haftada hangi gün kaç saat dersi
        # (seçilen seçeneğe göre ders veya ek ders) olduğunu gösteren
        # dictionarylerden oluşan liste, seçilen yıl ve ay bir dönemi kapsıyorsa
        # bir dict, iki dönemi kapsıyorsa iki dict döner.
        ders_etkinlik_list = DU.okutman_etkinlikleri_hesaplama(donem_list, okutman, kontrol)

        # Bulunan tarih araliklarina, okutmanın aylık ders etkinliklerine göre
        # ayın hangi gününde dersi varsa kaç saat dersi olduğu bilgisi,
        # izinliyse İ, resmi tatilse R bilgisini bir dictionary e doldurur.
        okutman_aylik_plan, ders_saati = DU.ders_saati_doldur(donem_list, ders_etkinlik_list,
                                                              resmi_tatil_list, personel_izin_list,
                                                              tarih_araligi, yil, ay)

        data_list['Öğretim Elemanı'] = okutman_adi

        for gun in range(1, takvim[1] + 1):
            if gun in okutman_aylik_plan:
                data_list[' ' + str(gun)] = str(okutman_aylik_plan[gun])
            else:
                data_list[' ' + str(gun)] = ' '

        data_list[ders_saati_turu] = str(ders_saati)

        item = {
            "type": "table-multiRow",
            "fields": data_list,
            "actions": False,
            'key': okutman.key
        }

        self.output['objects'].append(item)

        _form.pdf_sec = fields.Button(_(u"Pdf Çıkar"))
        _form.help_text = _(u"""
                         R: Resmi Tatil
                         İ: İzinli""")
        self.form_out(_form)
